@@ -1,65 +1,68 @@
 # SPDX-FileCopyrightText: 2022 James R. Barlow
 # SPDX-License-Identifier: MPL-2.0
 
-"""Interface to Ghostscript executable"""
+"""Interface to Ghostscript executable."""
 
 from __future__ import annotations
 
 import logging
 import os
 import re
-import sys
 from io import BytesIO
 from os import fspath
 from pathlib import Path
 from subprocess import PIPE, CalledProcessError
 
+from packaging.version import Version
 from PIL import Image, UnidentifiedImageError
 
 from ocrmypdf.exceptions import SubprocessOutputError
 from ocrmypdf.helpers import Resolution
 from ocrmypdf.subprocess import get_version, run, run_polling_stderr
 
-# Remove this workaround when we require Pillow >= 10
-try:
-    Transpose = Image.Transpose  # type: ignore
-except AttributeError:
-    # Pillow 9 shim
-    Transpose = Image  # type: ignore
+COLOR_CONVERSION_STRATEGIES = frozenset(
+    [
+        'CMYK',
+        'Gray',
+        'LeaveColorUnchanged',
+        'RGB',
+        'UseDeviceIndependentColor',
+    ]
+)
 
 log = logging.getLogger(__name__)
 
-# Most reliable what to get the bitness of Python interpreter, according to Python docs
-_IS_64BIT = sys.maxsize > 2**32
 
-_GSWIN = None
-if os.name == 'nt':
-    if _IS_64BIT:
-        _GSWIN = 'gswin64c'
-    else:
-        _GSWIN = 'gswin32c'
+class DuplicateFilter(logging.Filter):
+    """Filter out duplicate log messages."""
 
-GS = _GSWIN if _GSWIN else 'gs'
-del _GSWIN
+    def __init__(self, logger: logging.Logger):
+        self.last: logging.LogRecord | None = None
+        self.count = 0
+        self.logger = logger
+
+    def filter(self, record):
+        if self.last and record.msg == self.last.msg:
+            self.count += 1
+            return False
+        else:
+            if self.count >= 1:
+                rep_msg = f"(previous message repeated {self.count} times)"
+                self.count = 0  # Avoid infinite recursion
+                self.logger.log(self.last.levelno, rep_msg)
+            self.last = record
+            return True
 
 
-def version():
-    return get_version(GS)
+log.addFilter(DuplicateFilter(log))
 
 
-def jpeg_passthrough_available() -> bool:
-    """Returns True if the installed version of Ghostscript supports JPEG passthru
+# Ghostscript executable - gswin32c is not supported
+GS = 'gswin64c' if os.name == 'nt' else 'gs'
 
-    Prior to 9.23, Ghostscript decoded and re-encoded JPEGs internally. In 9.23
-    it gained the ability to keep JPEGs unmodified. However, the 9.23
-    implementation was buggy and would deletes the last two bytes of images in
-    some cases, as reported here.
-    https://bugs.ghostscript.com/show_bug.cgi?id=699216
 
-    The issue was fixed for 9.24, hence that is the first version we consider
-    the feature available. (Ghostscript 9.24 has its own problems is blacklisted.)
-    """
-    return version() >= '9.24'
+def version() -> Version:
+    return Version(get_version(GS))
 
 
 def _gs_error_reported(stream) -> bool:
@@ -77,6 +80,7 @@ def rasterize_pdf(
     page_dpi: Resolution | None = None,
     rotation: int | None = None,
     filter_vector: bool = False,
+    stop_on_error: bool = False,
 ):
     """Rasterize one page of a PDF at resolution raster_dpi in canvas units."""
     raster_dpi = raster_dpi.round(6)
@@ -97,6 +101,7 @@ def rasterize_pdf(
             f'-r{raster_dpi.x:f}x{raster_dpi.y:f}',
         ]
         + (['-dFILTERVECTOR'] if filter_vector else [])
+        + (['-dPDFSTOPONERROR'] if stop_on_error else [])
         + [
             '-o',
             '-',
@@ -124,11 +129,11 @@ def rasterize_pdf(
                 # rotation is a clockwise angle and Image.ROTATE_* is
                 # counterclockwise so this cancels out the rotation
                 if rotation == 90:
-                    im = im.transpose(Transpose.ROTATE_90)
+                    im = im.transpose(Image.Transpose.ROTATE_90)
                 elif rotation == 180:
-                    im = im.transpose(Transpose.ROTATE_180)
+                    im = im.transpose(Image.Transpose.ROTATE_180)
                 elif rotation == 270:
-                    im = im.transpose(Transpose.ROTATE_270)
+                    im = im.transpose(Image.Transpose.ROTATE_270)
                 if rotation % 180 == 90:
                     page_dpi = page_dpi.flip_axis()
             im.save(fspath(output_file), dpi=page_dpi)
@@ -172,9 +177,11 @@ def generate_pdfa(
     output_file: os.PathLike,
     *,
     compression: str,
+    color_conversion_strategy: str,
     pdf_version: str = '1.5',
     pdfa_part: str = '2',
     progressbar_class=None,
+    stop_on_error: bool = False,
 ):
     # Ghostscript's compression is all or nothing. We can either force all images
     # to JPEG, force all to Flate/PNG, or let it decide how to encode the images.
@@ -200,23 +207,16 @@ def generate_pdfa(
             "-dAutoFilterGrayImages=true",
         ]
 
-    strategy = 'LeaveColorUnchanged'
-    # Older versions of Ghostscript expect a leading slash in
-    # sColorConversionStrategy, newer ones should not have it. See Ghostscript
-    # git commit fe1c025d.
     gs_version = version()
-    strategy = ('/' + strategy) if gs_version < '9.19' else strategy
-
-    if gs_version == '9.23':
-        # 9.23: added JPEG passthrough as a new feature, but with a bug that
-        # incorrectly formats some images. Fixed as of 9.24. So we disable this
-        # feature for 9.23.
-        # https://bugs.ghostscript.com/show_bug.cgi?id=699216
-        compression_args.append('-dPassThroughJPEGImages=false')
-    elif gs_version == '9.56.0':
-        # 9.56.0 breaks our OCR...?
+    if gs_version == Version('9.56.0'):
+        # 9.56.0 breaks our OCR, should be fixed in 9.56.1
+        # https://bugs.ghostscript.com/show_bug.cgi?id=705187
         compression_args.append('-dNEWPDF=false')
 
+    if os.name == 'nt':
+        # Windows has lots of fatal "permission denied" errors
+        stop_on_error = False
+
     # nb no need to specify ProcessColorModel when ColorConversionStrategy
     # is set; see:
     # https://bugs.ghostscript.com/show_bug.cgi?id=699392
@@ -226,15 +226,16 @@ def generate_pdfa(
             "-dBATCH",
             "-dNOPAUSE",
             "-dSAFER",
-            "-dCompatibilityLevel=" + str(pdf_version),
+            f"-dCompatibilityLevel={str(pdf_version)}",
             "-sDEVICE=pdfwrite",
             "-dAutoRotatePages=/None",
-            "-sColorConversionStrategy=" + strategy,
+            f"-sColorConversionStrategy={color_conversion_strategy}",
         ]
+        + (['-dPDFSTOPONERROR'] if stop_on_error else [])
         + compression_args
         + [
             "-dJPEGQ=95",
-            "-dPDFA=" + pdfa_part,
+            f"-dPDFA={pdfa_part}",
             "-dPDFACompatibilityPolicy=1",
             "-o",
             "-",
@@ -265,14 +266,9 @@ def generate_pdfa(
         # If there is an error we log the whole stderr, except for filtering
         # duplicates.
         if _gs_error_reported(stderr):
-            last_part = None
-            repcount = 0
+            # Ghostscript outputs the pattern **** Error: ....  frequently.
+            # Occasionally the error message is spammed many times. We filter
+            # out duplicates of this message using the filter above. We use
+            # the **** pattern to split the stderr into parts.
             for part in stderr.split('****'):
-                if part != last_part:
-                    if repcount > 1:
-                        log.error(f"(previous error message repeated {repcount} times)")
-                        repcount = 0
-                    log.error(part)
-                else:
-                    repcount += 1
-                last_part = part
+                log.error(part)
