@@ -1,25 +1,24 @@
-# © 2018 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+"""Interface to pngquant executable."""
 
-"""Interface to pngquant executable"""
+from __future__ import annotations
 
 from contextlib import contextmanager
 from io import BytesIO
 from pathlib import Path
 from subprocess import PIPE
 
+from packaging.version import Version
 from PIL import Image
 
 from ocrmypdf.exceptions import MissingDependencyError
 from ocrmypdf.subprocess import get_version, run
 
 
-def version():
-    return get_version('pngquant', regex=r'(\d+(\.\d+)*).*')
+def version() -> Version:
+    return Version(get_version('pngquant', regex=r'(\d+(\.\d+)*).*'))
 
 
 def available():
