@@ -9,9 +9,9 @@ import os
 import shutil
 import sys
 from argparse import Namespace
+from collections.abc import Iterator
 from copy import copy
 from pathlib import Path
-from typing import Iterator
 
 from pluggy import PluginManager
 
@@ -59,7 +59,7 @@ class PdfContext:
 class PageContext:
     """Holds our context for a page.
 
-    Must be pickable, so stores only intrinsic/simple data elements or those
+    Must be pickle-able, so stores only intrinsic/simple data elements or those
     capable of their serializing themselves via ``__getstate__``.
     """
 
