@@ -10,11 +10,12 @@ import logging
 import os
 import re
 import sys
+from collections.abc import Iterable, Iterator, Sequence
 from contextlib import suppress
 from datetime import datetime, timezone
 from pathlib import Path
 from shutil import copyfileobj
-from typing import Any, Iterable, Sequence
+from typing import Any, BinaryIO, TypeVar, cast
 
 import img2pdf
 import pikepdf
@@ -27,6 +28,7 @@ from ocrmypdf._jobcontext import PageContext, PdfContext
 from ocrmypdf._version import PROGRAM_NAME
 from ocrmypdf._version import __version__ as VERSION
 from ocrmypdf.exceptions import (
+    DigitalSignatureError,
     DpiError,
     EncryptedPdfError,
     InputFileError,
@@ -37,20 +39,29 @@ from ocrmypdf.helpers import IMG2PDF_KWARGS, Resolution, safe_symlink
 from ocrmypdf.hocrtransform import HocrTransform
 from ocrmypdf.pdfa import generate_pdfa_ps
 from ocrmypdf.pdfinfo import Colorspace, Encoding, PageInfo, PdfInfo
+from ocrmypdf.pluginspec import OrientationConfidence
 
-# Remove this workaround when we require Pillow >= 10
-try:
-    BICUBIC = Image.Resampling.BICUBIC  # type: ignore
-except AttributeError:  # pragma: no cover
-    # Pillow 9 shim
-    BICUBIC = Image.BICUBIC  # type: ignore
-
+T = TypeVar("T")
 log = logging.getLogger(__name__)
 
 VECTOR_PAGE_DPI = 400
 
 
 def triage_image_file(input_file: Path, output_file: Path, options) -> None:
+    """Triage the input image file.
+
+    If the input file is an image, check its resolution and convert it to PDF.
+
+    Args:
+        input_file: The path to the input file.
+        output_file: The path to the output file.
+        options: An object containing the options passed to the OCRmyPDF command.
+
+    Raises:
+        UnsupportedImageFormatError: If the input file is not a supported image format.
+        DpiError: If the input image has no resolution (DPI) in its metadata or if the
+            resolution is not credible.
+    """
     log.info("Input file is not a PDF, checking if it is an image...")
     try:
         im = Image.open(input_file)
@@ -65,34 +76,32 @@ def triage_image_file(input_file: Path, output_file: Path, options) -> None:
             if im.info['dpi'] <= (96, 96) and not options.image_dpi:
                 log.info("Image size: (%d, %d)", *im.size)
                 log.info("Image resolution: (%d, %d)", *im.info['dpi'])
-                log.error(
+                raise DpiError(
                     "Input file is an image, but the resolution (DPI) is "
                     "not credible.  Estimate the resolution at which the "
                     "image was scanned and specify it using --image-dpi."
                 )
-                raise DpiError()
         elif not options.image_dpi:
             log.info("Image size: (%d, %d)", *im.size)
-            log.error(
+            raise DpiError(
                 "Input file is an image, but has no resolution (DPI) "
                 "in its metadata.  Estimate the resolution at which "
                 "image was scanned and specify it using --image-dpi."
             )
-            raise DpiError()
 
         if im.mode in ('RGBA', 'LA'):
-            log.error(
+            raise UnsupportedImageFormatError(
                 "The input image has an alpha channel. Remove the alpha "
                 "channel first."
             )
-            raise UnsupportedImageFormatError()
 
         if 'iccprofile' not in im.info:
             if im.mode == 'RGB':
                 log.info("Input image has no ICC profile, assuming sRGB")
             elif im.mode == 'CMYK':
-                log.error("Input CMYK image has no ICC profile, not usable")
-                raise UnsupportedImageFormatError()
+                raise UnsupportedImageFormatError(
+                    "Input CMYK image has no ICC profile, not usable"
+                )
 
     try:
         log.info("Image seems valid. Try converting to PDF...")
@@ -110,7 +119,6 @@ def triage_image_file(input_file: Path, output_file: Path, options) -> None:
             )
         log.info("Successfully converted to PDF, processing...")
     except img2pdf.ImageOpenError as e:
-        log.error(e)
         raise UnsupportedImageFormatError() from e
 
 
@@ -121,7 +129,6 @@ def _pdf_guess_version(input_file: Path, search_window=1024) -> str:
 
     Returns empty string if not found, indicating file is probably not PDF.
     """
-
     with open(input_file, 'rb') as f:
         signature = f.read(search_window)
     m = re.search(br'%PDF-(\d\.\d)', signature)
@@ -133,6 +140,7 @@ def _pdf_guess_version(input_file: Path, search_window=1024) -> str:
 def triage(
     original_filename: str, input_file: Path, output_file: Path, options
 ) -> Path:
+    """Triage the input file. We can handle PDFs and images."""
     try:
         if _pdf_guess_version(input_file):
             if options.image_dpi:
@@ -161,6 +169,7 @@ def get_pdfinfo(
     max_workers: int | None = None,
     check_pages=None,
 ) -> PdfInfo:
+    """Get the PDF info."""
     try:
         return PdfInfo(
             input_file,
@@ -177,31 +186,26 @@ def get_pdfinfo(
 
 
 def validate_pdfinfo_options(context: PdfContext) -> None:
+    """Validate the PDF info options."""
     pdfinfo = context.pdfinfo
     options = context.options
 
     if pdfinfo.needs_rendering:
-        log.error(
+        raise InputFileError(
             "This PDF contains dynamic XFA forms created by Adobe LiveCycle "
             "Designer and can only be read by Adobe Acrobat or Adobe Reader."
         )
-        raise InputFileError()
-    if pdfinfo.has_userunit and options.output_type.startswith('pdfa'):
-        log.error(
-            "This input file uses a PDF feature that is not supported "
-            "by Ghostscript, so you cannot use --output-type=pdfa for this "
-            "file. (Specifically, it uses the PDF-1.6 /UserUnit feature to "
-            "support very large or small page sizes, and Ghostscript cannot "
-            "output these files.)  Use --output-type=pdf instead."
-        )
-        raise InputFileError()
+    if pdfinfo.has_signature:
+        if options.invalidate_digital_signatures:
+            log.warning("All digital signatures will be invalidated")
+        else:
+            raise DigitalSignatureError()
     if pdfinfo.has_acroform:
         if options.redo_ocr:
-            log.error(
+            raise InputFileError(
                 "This PDF has a user fillable form. --redo-ocr is not "
                 "currently possible on such files."
             )
-            raise InputFileError()
         else:
             log.warning(
                 "This PDF has a fillable form. "
@@ -218,28 +222,23 @@ def validate_pdfinfo_options(context: PdfContext) -> None:
 
 
 def _vector_page_dpi(pageinfo: PageInfo) -> int:
+    """Get a DPI to use for vector pages, if the page has vector content."""
     return VECTOR_PAGE_DPI if pageinfo.has_vector or pageinfo.has_text else 0
 
 
-def get_page_dpi(pageinfo: PageInfo, options) -> Resolution:
-    "Get the DPI when nonsquare DPI is tolerable"
-    xres = max(
-        pageinfo.dpi.x or VECTOR_PAGE_DPI,
-        options.oversample or 0.0,
-        _vector_page_dpi(pageinfo),
-    )
-    yres = max(
-        pageinfo.dpi.y or VECTOR_PAGE_DPI,
-        options.oversample or 0,
-        _vector_page_dpi(pageinfo),
-    )
-    return Resolution(float(xres), float(yres))
+def get_page_square_dpi(
+    page_context: PageContext, image_dpi: Resolution | None = None
+) -> Resolution:
+    """Get the DPI when we require xres == yres, scaled to physical units.
 
-
-def get_page_square_dpi(pageinfo: PageInfo, options) -> Resolution:
-    "Get the DPI when we require xres == yres, scaled to physical units"
-    xres = pageinfo.dpi.x or 0.0
-    yres = pageinfo.dpi.y or 0.0
+    Page DPI includes UserUnit scaling.
+    """
+    pageinfo = page_context.pageinfo
+    options = page_context.options
+    if not image_dpi:
+        image_dpi = pageinfo.dpi
+    xres = image_dpi.x or 0.0
+    yres = image_dpi.y or 0.0
     userunit = float(pageinfo.userunit) or 1.0
     units = float(
         max(
@@ -252,12 +251,23 @@ def get_page_square_dpi(pageinfo: PageInfo, options) -> Resolution:
     return Resolution(units, units)
 
 
-def get_canvas_square_dpi(pageinfo: PageInfo, options) -> Resolution:
-    """Get the DPI when we require xres == yres, in Postscript units"""
+def get_canvas_square_dpi(
+    page_context: PageContext, image_dpi: Resolution | None = None
+) -> Resolution:
+    """Get the DPI when we require xres == yres, in Postscript units.
+
+    Canvas DPI is independent of PDF UserUnit scaling, which is
+    used to describe situations where the PDF user space is not 1:1 with
+    the physical units of the page.
+    """
+    pageinfo = page_context.pageinfo
+    options = page_context.options
+    if not image_dpi:
+        image_dpi = pageinfo.dpi
     units = float(
         max(
-            (pageinfo.dpi.x) or VECTOR_PAGE_DPI,
-            (pageinfo.dpi.y) or VECTOR_PAGE_DPI,
+            image_dpi.x or VECTOR_PAGE_DPI,
+            image_dpi.y or VECTOR_PAGE_DPI,
             _vector_page_dpi(pageinfo),
             options.oversample or 0.0,
         )
@@ -266,6 +276,7 @@ def get_canvas_square_dpi(pageinfo: PageInfo, options) -> Resolution:
 
 
 def is_ocr_required(page_context: PageContext) -> bool:
+    """Check if the page needs to be OCR'd."""
     pageinfo = page_context.pageinfo
     options = page_context.options
 
@@ -315,8 +326,8 @@ def is_ocr_required(page_context: PageContext) -> bool:
             log.warning(
                 "page has no images - "
                 "all vector content will be "
-                f"rasterized at {VECTOR_PAGE_DPI} DPI, losing some resolution and likely "
-                "increasing file size. Use --oversample to adjust the "
+                f"rasterized at {VECTOR_PAGE_DPI} DPI, losing some resolution and "
+                "likely increasing file size. Use --oversample to adjust the "
                 "DPI."
             )
         else:
@@ -341,9 +352,12 @@ def is_ocr_required(page_context: PageContext) -> bool:
 
 
 def rasterize_preview(input_file: Path, page_context: PageContext) -> Path:
+    """Generate a lower quality preview image."""
     output_file = page_context.get_path('rasterize_preview.jpg')
-    canvas_dpi = get_canvas_square_dpi(page_context.pageinfo, page_context.options)
-    page_dpi = get_page_square_dpi(page_context.pageinfo, page_context.options)
+    canvas_dpi = Resolution(300.0, 300.0).take_min(
+        [get_canvas_square_dpi(page_context)]
+    )
+    page_dpi = Resolution(300.0, 300.0).take_min([get_page_square_dpi(page_context)])
     page_context.plugin_manager.hook.rasterize_pdf_page(
         input_file=input_file,
         output_file=output_file,
@@ -353,14 +367,15 @@ def rasterize_preview(input_file: Path, page_context: PageContext) -> Path:
         page_dpi=page_dpi,
         rotation=0,
         filter_vector=False,
+        stop_on_soft_error=not page_context.options.continue_on_soft_render_error,
     )
     return output_file
 
 
-def describe_rotation(page_context: PageContext, orient_conf, correction: int) -> str:
-    """
-    Describe the page rotation we are going to perform.
-    """
+def describe_rotation(
+    page_context: PageContext, orient_conf: OrientationConfidence, correction: int
+) -> str:
+    """Describe the page rotation we are going to perform (or not perform)."""
     direction = {0: '⇧', 90: '⇨', 180: '⇩', 270: '⇦'}
     turns = {0: ' ', 90: '⬏', 180: '↻', 270: '⬑'}
 
@@ -387,7 +402,7 @@ def describe_rotation(page_context: PageContext, orient_conf, correction: int) -
 
 
 def get_orientation_correction(preview: Path, page_context: PageContext) -> int:
-    """Work out orientation correct for each page.
+    """Work out orientation correction for each page.
 
     We ask Ghostscript to draw a preview page, which will rasterize with the
     current /Rotate applied, and then ask OCR which way the page is
@@ -401,7 +416,6 @@ def get_orientation_correction(preview: Path, page_context: PageContext) -> int:
     which points it (hopefully) upright. _graft.py takes care of the orienting
     the image and text layers.
     """
-
     orient_conf = page_context.plugin_manager.hook.get_ocr_engine().get_orientation(
         preview, page_context.options
     )
@@ -417,13 +431,59 @@ def get_orientation_correction(preview: Path, page_context: PageContext) -> int:
     return 0
 
 
+def calculate_image_dpi(page_context: PageContext) -> Resolution:
+    """Calculate the DPI for the page image."""
+    pageinfo = page_context.pageinfo
+    dpi_profile = pageinfo.page_dpi_profile()
+    if dpi_profile and dpi_profile.average_to_max_dpi_ratio < 0.8:
+        image_dpi = Resolution(dpi_profile.weighted_dpi, dpi_profile.weighted_dpi)
+    else:
+        image_dpi = pageinfo.dpi
+    return image_dpi
+
+
+def calculate_raster_dpi(page_context: PageContext):
+    """Calculate the DPI for rasterization."""
+    # Produce the page image with square resolution or else deskew and OCR
+    # will not work properly.
+    image_dpi = calculate_image_dpi(page_context)
+    dpi_profile = page_context.pageinfo.page_dpi_profile()
+    canvas_dpi = get_canvas_square_dpi(page_context, image_dpi)
+    page_dpi = get_page_square_dpi(page_context, image_dpi)
+    if dpi_profile and dpi_profile.average_to_max_dpi_ratio < 0.8:
+        log.warning(
+            "Weight average image DPI is %0.1f, max DPI is %0.1f. "
+            "The discrepancy may indicate a high detail region on this page, "
+            "but could also indicate a problem with the input PDF file. "
+            "Page image will be rendered at %0.1f DPI.",
+            dpi_profile.weighted_dpi,
+            dpi_profile.max_dpi,
+            canvas_dpi.to_scalar(),
+        )
+    return canvas_dpi, page_dpi
+
+
 def rasterize(
     input_file: Path,
     page_context: PageContext,
     correction: int = 0,
     output_tag: str = '',
-    remove_vectors=None,
-):
+    remove_vectors: bool | None = None,
+) -> Path:
+    """Rasterize a PDF page to a PNG image.
+
+    Args:
+        input_file: The input PDF file path.
+        page_context: The page context object.
+        correction: The orientation correction angle. Defaults to 0.
+        output_tag: The output tag. Defaults to ''.
+        remove_vectors: Whether to remove vectors. Defaults to None, which means
+            the value from the page context options will be used. If the value
+            is True or False, it will override the page context options.
+
+    Returns:
+        Path: The output PNG file path.
+    """
     colorspaces = ['pngmono', 'pnggray', 'png256', 'png16m']
     device_idx = 0
 
@@ -454,10 +514,7 @@ def rasterize(
 
     log.debug(f"Rasterize with {device}, rotation {correction}")
 
-    # Produce the page image with square resolution or else deskew and OCR
-    # will not work properly.
-    canvas_dpi = get_canvas_square_dpi(pageinfo, page_context.options)
-    page_dpi = get_page_square_dpi(pageinfo, page_context.options)
+    canvas_dpi, page_dpi = calculate_raster_dpi(page_context)
 
     page_context.plugin_manager.hook.rasterize_pdf_page(
         input_file=input_file,
@@ -468,11 +525,13 @@ def rasterize(
         pageno=pageinfo.pageno + 1,
         rotation=correction,
         filter_vector=remove_vectors,
+        stop_on_soft_error=not page_context.options.continue_on_soft_render_error,
     )
     return output_file
 
 
 def preprocess_remove_background(input_file: Path, page_context: PageContext) -> Path:
+    """Remove the background from the input image (temporarily disabled)."""
     if any(image.bpc > 1 for image in page_context.pageinfo.images):
         raise NotImplementedError("--remove-background is temporarily not implemented")
         # output_file = page_context.get_path('pp_rm_bg.png')
@@ -483,8 +542,17 @@ def preprocess_remove_background(input_file: Path, page_context: PageContext) ->
 
 
 def preprocess_deskew(input_file: Path, page_context: PageContext) -> Path:
+    """Deskews the input image using the OCR engine and saves the output to a file.
+
+    Args:
+        input_file: The input image file to deskew.
+        page_context: The context of the page being processed.
+
+    Returns:
+        Path: The path to the deskewed image file.
+    """
     output_file = page_context.get_path('pp_deskew.png')
-    dpi = get_page_square_dpi(page_context.pageinfo, page_context.options)
+    dpi = get_page_square_dpi(page_context, calculate_image_dpi(page_context))
 
     ocr_engine = page_context.plugin_manager.hook.get_ocr_engine()
     deskew_angle_degrees = ocr_engine.get_deskew(input_file, page_context.options)
@@ -494,7 +562,7 @@ def preprocess_deskew(input_file: Path, page_context: PageContext) -> Path:
         # resampling if image is mode '1' or 'P'
         deskewed = im.rotate(
             deskew_angle_degrees,
-            resample=BICUBIC,
+            resample=Image.Resampling.BICUBIC,
             fillcolor=ImageColor.getcolor('white', mode=im.mode),  # type: ignore
         )
         deskewed.save(output_file, dpi=dpi)
@@ -503,21 +571,23 @@ def preprocess_deskew(input_file: Path, page_context: PageContext) -> Path:
 
 
 def preprocess_clean(input_file: Path, page_context: PageContext) -> Path:
+    """Clean the input image using unpaper."""
     output_file = page_context.get_path('pp_clean.png')
-    dpi = get_page_square_dpi(page_context.pageinfo, page_context.options)
+    dpi = get_page_square_dpi(page_context, calculate_image_dpi(page_context))
     return unpaper.clean(
         input_file,
         output_file,
-        dpi=dpi.x,
+        dpi=dpi.to_scalar(),
         unpaper_args=page_context.options.unpaper_args,
     )
 
 
 def create_ocr_image(image: Path, page_context: PageContext) -> Path:
-    """Create the image we send for OCR. May not be the same as the display
-    image depending on preprocessing. This image will never be shown to the
-    user."""
+    """Create the image we send for OCR.
 
+    Might not be the same as the display image depending on preprocessing.
+    This image will never be shown to the user.
+    """
     output_file = page_context.get_path('ocr.png')
     options = page_context.options
     with Image.open(image) as im:
@@ -562,6 +632,7 @@ def create_ocr_image(image: Path, page_context: PageContext) -> Path:
 
 
 def ocr_engine_hocr(input_file: Path, page_context: PageContext) -> tuple[Path, Path]:
+    """Run the OCR engine and generate hOCR output."""
     hocr_out = page_context.get_path('ocr_hocr.hocr')
     hocr_text_out = page_context.get_path('ocr_hocr.txt')
     options = page_context.options
@@ -577,11 +648,26 @@ def ocr_engine_hocr(input_file: Path, page_context: PageContext) -> tuple[Path,
 
 
 def should_visible_page_image_use_jpg(pageinfo: PageInfo) -> bool:
-    # If all images were JPEGs originally, produce a JPEG as output
-    return pageinfo.images and all(im.enc == Encoding.jpeg for im in pageinfo.images)
+    """Determines whether the visible page image should be saved as a JPEG.
+
+    If all images were JPEGs originally, permit a JPEG as output.
+
+    Args:
+        pageinfo: The PageInfo object containing information about the page.
+
+    Returns:
+        A boolean indicating whether the visible page image should be saved as a JPEG.
+    """
+    return bool(pageinfo.images) and all(
+        im.enc == Encoding.jpeg for im in pageinfo.images
+    )
 
 
 def create_visible_page_jpg(image: Path, page_context: PageContext) -> Path:
+    """Create a visible page image in JPEG format.
+
+    This is intended to be used when all images on the page were originally JPEGs.
+    """
     output_file = page_context.get_path('visible.jpg')
     with Image.open(image) as im:
         # At this point the image should be a .png, but deskew, unpaper
@@ -593,7 +679,7 @@ def create_visible_page_jpg(image: Path, page_context: PageContext) -> Path:
             dpi = Resolution(*im.info['dpi'])
         else:
             # Fallback to page-implied DPI
-            dpi = get_page_square_dpi(page_context.pageinfo, page_context.options)
+            dpi = get_page_square_dpi(page_context, calculate_image_dpi(page_context))
 
         # Pillow requires integer DPI
         im.save(output_file, format='JPEG', dpi=dpi.to_int())
@@ -603,6 +689,7 @@ def create_visible_page_jpg(image: Path, page_context: PageContext) -> Path:
 def create_pdf_page_from_image(
     image: Path, page_context: PageContext, orientation_correction: int
 ) -> Path:
+    """Create a PDF page from a page image."""
     # We rasterize a square DPI version of each page because most image
     # processing tools don't support rectangular DPI. Use the square DPI as it
     # accurately describes the image. It would be possible to resample the image
@@ -634,9 +721,10 @@ def create_pdf_page_from_image(
 
 
 def render_hocr_page(hocr: Path, page_context: PageContext) -> Path:
+    """Render the hOCR page to a PDF."""
     options = page_context.options
     output_file = page_context.get_path('ocr_hocr.pdf')
-    dpi = get_page_square_dpi(page_context.pageinfo, options)
+    dpi = get_page_square_dpi(page_context, calculate_image_dpi(page_context))
     debug_mode = options.pdf_renderer == 'hocrdebug'
 
     hocrtransform = HocrTransform(hocr_filename=hocr, dpi=dpi.x)  # square
@@ -653,6 +741,7 @@ def render_hocr_page(hocr: Path, page_context: PageContext) -> Path:
 def ocr_engine_textonly_pdf(
     input_image: Path, page_context: PageContext
 ) -> tuple[Path, Path]:
+    """Run the OCR engine and generate a text-only PDF (will look blank)."""
     output_pdf = page_context.get_path('ocr_tess.pdf')
     output_text = page_context.get_path('ocr_tess.txt')
     options = page_context.options
@@ -668,6 +757,7 @@ def ocr_engine_textonly_pdf(
 
 
 def get_docinfo(base_pdf: pikepdf.Pdf, context: PdfContext) -> dict[str, str]:
+    """Read the document info and store it in a dictionary."""
     options = context.options
 
     def from_document_info(key):
@@ -699,12 +789,28 @@ def get_docinfo(base_pdf: pikepdf.Pdf, context: PdfContext) -> dict[str, str]:
 
 
 def generate_postscript_stub(context: PdfContext) -> Path:
+    """Generates a PostScript file stub for the given PDF context.
+
+    Args:
+        context: The PDF context to generate the PostScript file stub for.
+
+    Returns:
+        Path: The path to the generated PostScript file stub.
+    """
     output_file = context.get_path('pdfa.ps')
     generate_pdfa_ps(output_file)
     return output_file
 
 
 def convert_to_pdfa(input_pdf: Path, input_ps_stub: Path, context: PdfContext) -> Path:
+    """Converts the given PDF to PDF/A.
+
+    Args:
+        input_pdf: The input PDF file path (presumably not PDF/A).
+        input_ps_stub: The input PostScript file path, containing instructions
+            for the PDF/A generator to use.
+        context: The PDF context.
+    """
     options = context.options
     input_pdfinfo = context.pdfinfo
     fix_docinfo_file = context.get_path('fix_docinfo.pdf')
@@ -715,21 +821,8 @@ def convert_to_pdfa(input_pdf: Path, input_ps_stub: Path, context: PdfContext) -
     # NULs in DocumentInfo seem to be common since older Acrobats included them.
     # pikepdf can deal with this, but we make the world a better place by
     # stamping them out as soon as possible.
-    modified = False
     with pikepdf.open(input_pdf) as pdf_file:
-        try:
-            len(pdf_file.docinfo)
-        except TypeError:
-            log.error(
-                "File contains a malformed DocumentInfo block - continuing anyway"
-            )
-        else:
-            if pdf_file.docinfo:
-                for k, v in pdf_file.docinfo.items():
-                    if b'\x00' in bytes(v):
-                        pdf_file.docinfo[k] = bytes(v).replace(b'\x00', b'')
-                        modified = True
-        if modified:
+        if _repair_docinfo_nuls(pdf_file):
             pdf_file.save(fix_docinfo_file)
         else:
             safe_symlink(input_pdf, fix_docinfo_file)
@@ -739,19 +832,43 @@ def convert_to_pdfa(input_pdf: Path, input_ps_stub: Path, context: PdfContext) -
         pdf_pages=[fix_docinfo_file],
         pdfmark=input_ps_stub,
         output_file=output_file,
-        compression=options.pdfa_image_compression,
+        context=context,
         pdfa_part=options.output_type[-1],  # is pdfa-1, pdfa-2, or pdfa-3
         progressbar_class=(
             context.plugin_manager.hook.get_progressbar_class()
             if options.progress_bar
             else None
         ),
+        stop_on_soft_error=not options.continue_on_soft_render_error,
     )
 
     return output_file
 
 
+def _repair_docinfo_nuls(pdf):
+    """If the DocumentInfo block contains NUL characters, remove them.
+
+    If the DocumentInfo block is malformed, log an error and continue.
+    """
+    modified = False
+    try:
+        if not isinstance(pdf.docinfo, pikepdf.Dictionary):
+            raise TypeError("DocumentInfo is not a dictionary")
+        for k, v in pdf.docinfo.items():
+            if isinstance(v, str) and b'\x00' in bytes(v):
+                pdf.docinfo[k] = bytes(v).replace(b'\x00', b'')
+                modified = True
+    except TypeError:
+        # TypeError can also be raised if dictionary items are unexpected types
+        log.error("File contains a malformed DocumentInfo block - continuing anyway.")
+    return modified
+
+
 def should_linearize(working_file: Path, context: PdfContext) -> bool:
+    """Determine whether the PDF should be linearized.
+
+    For smaller files, linearization is not worth the effort.
+    """
     filesize = os.stat(working_file).st_size
     if filesize > (context.options.fast_web_view * 1_000_000):
         return True
@@ -759,6 +876,11 @@ def should_linearize(working_file: Path, context: PdfContext) -> bool:
 
 
 def get_pdf_save_settings(output_type: str) -> dict[str, Any]:
+    """Get pikepdf.Pdf.save settings for the given output type.
+
+    Essentially, don't use features that are incompatible with a given
+    PDF/A specification.
+    """
     if output_type == 'pdfa-1':
         # Trigger recompression to ensure object streams are removed, because
         # Acrobat complains about them in PDF/A-1b validation.
@@ -777,6 +899,11 @@ def get_pdf_save_settings(output_type: str) -> dict[str, Any]:
 
 
 def metadata_fixup(working_file: Path, context: PdfContext) -> Path:
+    """Fix certain metadata fields after Ghostscript PDF/A conversion.
+
+    Also report on metadata in the input file that was not retained during
+    PDF/A conversion.
+    """
     output_file = context.get_path('metafix.pdf')
     options = context.options
 
@@ -799,24 +926,43 @@ def metadata_fixup(working_file: Path, context: PdfContext) -> Path:
 
     with pikepdf.open(context.origin) as original, pikepdf.open(working_file) as pdf:
         docinfo = get_docinfo(original, context)
-        with pdf.open_metadata() as meta:
-            meta.load_from_docinfo(docinfo, delete_missing=False, raise_failure=False)
+        with original.open_metadata(
+            set_pikepdf_as_editor=False, update_docinfo=False, strict=False
+        ) as meta_original, pdf.open_metadata() as meta_pdf:
+            meta_pdf.load_from_docinfo(
+                docinfo, delete_missing=False, raise_failure=False
+            )
             # If xmp:CreateDate is missing, set it to the modify date to
-            # match Ghostscript, for consistency
-            if 'xmp:CreateDate' not in meta:
-                meta['xmp:CreateDate'] = meta.get('xmp:ModifyDate', '')
-
-            with original.open_metadata(
-                set_pikepdf_as_editor=False, update_docinfo=False, strict=False
-            ) as meta_original:
-                if meta.get('dc:title') == 'Untitled':
-                    # Ghostscript likes to set title to Untitled if omitted from input.
-                    # Reverse this, because PDF/A TechNote 0003:Metadata in PDF/A-1
-                    # and the XMP Spec do not make this recommendation.
-                    if 'dc:title' not in meta_original:
-                        del meta['dc:title']
-                missing = set(meta_original.keys()) - set(meta.keys())
-                report_on_metadata(missing)
+            # ensure consistency with Ghostscript.
+            if 'xmp:CreateDate' not in meta_pdf:
+                meta_pdf['xmp:CreateDate'] = meta_pdf.get('xmp:ModifyDate', '')
+            if meta_pdf.get('dc:title') == 'Untitled':
+                # Ghostscript likes to set title to Untitled if omitted from input.
+                # Reverse this, because PDF/A TechNote 0003:Metadata in PDF/A-1
+                # and the XMP Spec do not make this recommendation.
+                if 'dc:title' not in meta_original:
+                    del meta_pdf['dc:title']
+            # If the user explicitly specified an empty string for any of the
+            # following, they should be unset and not reported as missing in
+            # the output pdf. Note that some metadata fields use differing names
+            # between PDF-A and PDF.
+            for meta in [meta_pdf, meta_original]:
+                if options.title == '' and 'dc:title' in meta:
+                    del meta['dc:title']  # PDF-A and PDF
+                if options.author == '':
+                    if 'dc:creator' in meta:
+                        del meta['dc:creator']  # PDF-A (Not xmp:CreatorTool)
+                    if 'pdf:Author' in meta:
+                        del meta['pdf:Author']  # PDF
+                if options.subject == '':
+                    if 'dc:description' in meta:
+                        del meta['dc:description']  # PDF-A
+                    if 'dc:subject' in meta:
+                        del meta['dc:subject']  # PDF
+                if options.keywords == '' and 'pdf:Keywords' in meta:
+                    del meta['pdf:Keywords']  # PDF-A and PDF
+            meta_missing = set(meta_original.keys()) - set(meta_pdf.keys())
+            report_on_metadata(meta_missing)
 
         optimizing = context.plugin_manager.hook.is_optimization_enabled(
             context=context
@@ -832,9 +978,33 @@ def metadata_fixup(working_file: Path, context: PdfContext) -> Path:
     return output_file
 
 
+def _file_size_ratio(
+    input_file: Path, output_file: Path
+) -> tuple[float | None, float | None]:
+    """Calculate ratio of input to output file sizes and percentage savings.
+
+    Args:
+        input_file (Path): The path to the input file.
+        output_file (Path): The path to the output file.
+
+    Returns:
+        tuple[float | None, float | None]: A tuple containing the file size
+        ratio and the percentage savings achieved by the output file size
+        compared to the input file size.
+    """
+    input_size = input_file.stat().st_size
+    output_size = output_file.stat().st_size
+    if output_size == 0:
+        return None, None
+    ratio = input_size / output_size
+    savings = 1 - output_size / input_size
+    return ratio, savings
+
+
 def optimize_pdf(
     input_file: Path, context: PdfContext, executor: Executor
 ) -> tuple[Path, Sequence[str]]:
+    """Optimize the given PDF file."""
     output_file = context.get_path('optimize.pdf')
     output_pdf, messages = context.plugin_manager.hook.optimize_pdf(
         input_pdf=input_file,
@@ -844,17 +1014,29 @@ def optimize_pdf(
         linearize=should_linearize(input_file, context),
     )
 
-    input_size = input_file.stat().st_size
-    output_size = output_file.stat().st_size
-    if output_size > 0:
-        ratio = input_size / output_size
-        savings = 1 - output_size / input_size
-        log.info(f"Optimize ratio: {ratio:.2f} savings: {(savings):.1%}")
-
+    ratio, savings = _file_size_ratio(input_file, output_file)
+    if ratio:
+        log.info(f"Image optimization ratio: {ratio:.2f} savings: {(savings):.1%}")
+    ratio, savings = _file_size_ratio(context.origin, output_file)
+    if ratio:
+        log.info(f"Total file size ratio: {ratio:.2f} savings: {(savings):.1%}")
     return output_pdf, messages
 
 
-def enumerate_compress_ranges(iterable):
+def enumerate_compress_ranges(
+    iterable: Iterable[T],
+) -> Iterator[tuple[tuple[int, int], T]]:
+    """Enumerate the ranges of non-empty elements in an iterable.
+
+    Compresses consecutive ranges of length 1 into single elements.
+
+    Args:
+        iterable: An iterable of elements to enumerate.
+
+    Yields:
+        A tuple containing a range of indices and the corresponding element.
+        If the element is None, the range represents a skipped range of indices.
+    """
     skipped_from, index = None, None
     for index, txt_file in enumerate(iterable):
         index += 1
@@ -871,21 +1053,22 @@ def enumerate_compress_ranges(iterable):
 
 
 def merge_sidecars(txt_files: Iterable[Path | None], context: PdfContext) -> Path:
+    """Merge the page sidecar files into a single file.
+
+    Sidecar files are created by the OCR engine and contain the text for each
+    page in the PDF. This function merges the sidecar files into a single file
+    and returns the path to the merged file.
+    """
     output_file = context.get_path('sidecar.txt')
     with open(output_file, 'w', encoding="utf-8") as stream:
         for (from_, to_), txt_file in enumerate_compress_ranges(txt_files):
             if from_ != 1:
-                stream.write('\f')  # Form feed between pages
+                stream.write('\f')  # Form feed between pages for all pages after first
             if txt_file:
-                with open(txt_file, encoding="utf-8") as in_:
-                    txt = in_.read()
-                    # Some OCR engines (e.g. Tesseract v4 alpha) add form feeds
-                    # between pages, and some do not. For consistency, we ignore
-                    # any added by the OCR engine and them on our own.
-                    if txt.endswith('\f'):
-                        stream.write(txt[:-1])
-                    else:
-                        stream.write(txt)
+                txt = txt_file.read_text(encoding="utf-8")
+                # Some versions of Tesseract add a form feed at the end and
+                # others don't. Remove it if it exists, since we add one manually.
+                stream.write(txt.removesuffix('\f'))
             else:
                 if from_ != to_:
                     pages = f'{from_}-{to_}'
@@ -895,15 +1078,27 @@ def merge_sidecars(txt_files: Iterable[Path | None], context: PdfContext) -> Pat
     return output_file
 
 
-def copy_final(input_file, output_file, _context: PdfContext) -> None:
+def copy_final(
+    input_file: Path, output_file: str | Path | BinaryIO, _context: PdfContext
+) -> None:
+    """Copy the final temporary file to the output destination.
+
+    Args:
+        input_file (Path): The input file to copy.
+        output_file (str | Path | BinaryIO): The output file to copy to.
+        _context (PdfContext): The PDF context.
+
+    Returns:
+        None
+    """
     log.debug('%s -> %s', input_file, output_file)
-    with open(input_file, 'rb') as input_stream:
+    with input_file.open('rb') as input_stream:
         if output_file == '-':
-            copyfileobj(input_stream, sys.stdout.buffer)
+            copyfileobj(input_stream, sys.stdout.buffer)  # type: ignore[misc]
             sys.stdout.flush()
         elif hasattr(output_file, 'writable'):
-            output_stream = output_file
-            copyfileobj(input_stream, output_stream)
+            output_stream = cast(BinaryIO, output_file)
+            copyfileobj(input_stream, output_stream)  # type: ignore[misc]
             with suppress(AttributeError):
                 output_stream.flush()
         else:
