@@ -13,12 +13,13 @@ import logging.handlers
 import os
 import sys
 import threading
+from collections.abc import Sequence
 from concurrent.futures.process import BrokenProcessPool
 from concurrent.futures.thread import BrokenThreadPool
 from functools import partial
 from pathlib import Path
 from tempfile import mkdtemp
-from typing import NamedTuple, Sequence, cast
+from typing import NamedTuple, cast
 
 import PIL
 
@@ -104,6 +105,7 @@ def preprocess(
     deskew: bool,
     clean: bool,
 ) -> Path:
+    """Preprocess an image."""
     if remove_background:
         image = preprocess_remove_background(image, page_context)
     if deskew:
@@ -116,6 +118,7 @@ def preprocess(
 def make_intermediate_images(
     page_context: PageContext, orientation_correction: int
 ) -> tuple[Path, Path | None]:
+    """Create intermediate and preprocessed images for OCR."""
     options = page_context.options
 
     ocr_image = preprocess_out = None
@@ -173,6 +176,7 @@ def make_intermediate_images(
 
 
 def exec_page_sync(page_context: PageContext) -> PageResult:
+    """Execute a pipeline for a single page synchronously."""
     options = page_context.options
     tls.pageno = page_context.pageno + 1
 
@@ -233,6 +237,7 @@ def exec_page_sync(page_context: PageContext) -> PageResult:
 def post_process(
     pdf_file: Path, context: PdfContext, executor: Executor
 ) -> tuple[Path, Sequence[str]]:
+    """Postprocess the PDF file."""
     pdf_out = pdf_file
     if context.options.output_type.startswith('pdfa'):
         ps_stub_out = generate_postscript_stub(context)
@@ -243,6 +248,7 @@ def post_process(
 
 
 def worker_init(max_pixels: int) -> None:
+    """Initialize a worker thread or process."""
     # In Windows, child process will not inherit our change to this value in
     # the parent process, so ensure workers get it set. Not needed when running
     # threaded, but harmless to set again.
@@ -251,9 +257,8 @@ def worker_init(max_pixels: int) -> None:
 
 
 def exec_concurrent(context: PdfContext, executor: Executor) -> Sequence[str]:
-    """Execute the pipeline concurrently"""
-
-    # Run exec_page_sync on every page context
+    """Execute the OCR pipeline concurrently."""
+    # Run exec_page_sync on every page
     options = context.options
     max_workers = min(len(context.pdfinfo), options.jobs)
     if max_workers > 1:
@@ -263,6 +268,7 @@ def exec_concurrent(context: PdfContext, executor: Executor) -> Sequence[str]:
     ocrgraft = OcrGrafter(context)
 
     def update_page(result: PageResult, pbar):
+        """After OCR is complete for a page, update the PDF."""
         try:
             tls.pageno = result.pageno + 1
             sidecars[result.pageno] = result.text
@@ -316,10 +322,9 @@ def exec_concurrent(context: PdfContext, executor: Executor) -> Sequence[str]:
 def configure_debug_logging(
     log_filename: Path, prefix: str = ''
 ) -> logging.FileHandler:
-    """
-    Create a debug log file at a specified location.
+    """Create a debug log file at a specified location.
 
-    Arguments:
+    Args:
         log_filename: Where to the put the log file.
         prefix: The logging domain prefix that should be sent to the log.
     """
@@ -340,6 +345,17 @@ def run_pipeline(
     plugin_manager: OcrmypdfPluginManager | None,
     api: bool = False,
 ) -> ExitCode:
+    """Run the OCR pipeline.
+
+    Args:
+        options: The parsed command line options.
+        plugin_manager: The plugin manager to use. If not provided, one will be
+            created.
+        api: If ``True``, the pipeline is being run from the API. This is used
+            to manage exceptions in a way appropriate for API or CLI usage.
+            For CLI (``api=False``), exceptions are printed and described;
+            for API use, they are propagated to the caller.
+    """
     # Any changes to options will not take effect for options that are already
     # bound to function parameters in the pipeline. (For example
     # options.input_file, options.pdf_renderer are already bound.)
@@ -420,13 +436,13 @@ def run_pipeline(
                 options, start_input_file, options.output_file, optimize_messages
             )
 
-    except (KeyboardInterrupt if not api else NeverRaise):
+    except KeyboardInterrupt if not api else NeverRaise:
         if options.verbose >= 1:
             log.exception("KeyboardInterrupt")
         else:
             log.error("KeyboardInterrupt")
         return ExitCode.ctrl_c
-    except (ExitCodeException if not api else NeverRaise) as e:
+    except ExitCodeException if not api else NeverRaise as e:
         e = cast(ExitCodeException, e)
         if options.verbose >= 1:
             log.exception("ExitCodeException")
@@ -435,7 +451,7 @@ def run_pipeline(
         else:
             log.error(type(e).__name__)
         return e.exit_code
-    except (PIL.Image.DecompressionBombError if not api else NeverRaise):
+    except PIL.Image.DecompressionBombError if not api else NeverRaise:
         log.exception(
             "A decompression bomb error was encountered while executing the "
             "pipeline. Use the argument --max-image-mpixels to raise the maximum "
@@ -453,7 +469,7 @@ def run_pipeline(
             "argument."
         )
         return ExitCode.child_process_error
-    except (Exception if not api else NeverRaise):  # pylint: disable=broad-except
+    except Exception if not api else NeverRaise:  # pylint: disable=broad-except
         log.exception("An exception occurred while executing the pipeline")
         return ExitCode.other_error
     finally:
