@@ -12,9 +12,9 @@ import os
 import sys
 import unicodedata
 from argparse import Namespace
+from collections.abc import Sequence
 from pathlib import Path
 from shutil import copyfileobj
-from typing import Sequence
 
 import pikepdf
 import PIL
@@ -43,35 +43,47 @@ log = logging.getLogger(__name__)
 
 
 def check_platform() -> None:
-    if os.name == 'nt' and sys.maxsize <= 2**32:  # pragma: no cover
-        # 32-bit interpreter on Windows
-        log.error(
-            "You are running OCRmyPDF in a 32-bit (x86) Python interpreter."
+    if sys.maxsize <= 2**32:  # pragma: no cover
+        log.warning(
+            "You are running OCRmyPDF in a 32-bit (x86) Python interpreter. "
+            "This is not supported. 32-bit does not have enough address space "
+            "to process large files. "
             "Please use a 64-bit (x86-64) version of Python."
         )
 
 
-def check_options_languages(options: Namespace, ocr_engine_languages: set[str]) -> None:
+def check_options_languages(
+    options: Namespace, ocr_engine_languages: list[str]
+) -> None:
     if not options.languages:
-        options.languages = {DEFAULT_LANGUAGE}
+        options.languages = [DEFAULT_LANGUAGE]
         system_lang = locale.getlocale()[0]
         if system_lang and not system_lang.startswith('en'):
             log.debug("No language specified; assuming --language %s", DEFAULT_LANGUAGE)
     if not ocr_engine_languages:
         return
-    missing_languages = options.languages - ocr_engine_languages
+    missing_languages = set(options.languages) - set(ocr_engine_languages)
     if missing_languages:
+        lang_text = '\n'.join(lang for lang in missing_languages)
         msg = (
             "OCR engine does not have language data for the following "
             "requested languages: \n"
+            f"{lang_text}\n"
+            "Please install the appropriate language data for your OCR engine.\n"
+            "\n"
+            "See the online documentation for instructions:\n"
+            "    https://ocrmypdf.readthedocs.io/en/latest/languages.html\n"
+            "\n"
+            "Note: most languages are identified by a 3-letter ISO 639-2 Code.\n"
+            "For example, English is 'eng', German is 'deu', and Spanish is 'spa'.\n"
+            "Simplified Chinese is 'chi_sim' and Traditional Chinese is 'chi_tra'."
+            "\n"
         )
-        msg += '\n'.join(lang for lang in missing_languages)
-        msg += '\nNote: most languages are identified by a 3-digit ISO 639-2 Code'
         raise MissingDependencyError(msg)
 
 
 def check_options_output(options: Namespace) -> None:
-    is_latin = options.languages.issubset(HOCR_OK_LANGS)
+    is_latin = set(options.languages).issubset(HOCR_OK_LANGS)
 
     if options.pdf_renderer.startswith('hocr') and not is_latin:
         log.warning(
@@ -109,12 +121,10 @@ def check_options_output(options: Namespace) -> None:
 def check_options_sidecar(options: Namespace) -> None:
     if options.sidecar == '\0':
         if options.output_file == '-':
-            raise BadArgsError(
-                "--sidecar filename must be specified when output file is stdout."
-            )
+            raise BadArgsError("--sidecar filename needed when output file is stdout.")
         elif options.output_file == os.devnull:
             raise BadArgsError(
-                "--sidecar filename must be specified when output file is /dev/null or NUL."
+                "--sidecar filename needed when output file is /dev/null or NUL."
             )
         options.sidecar = options.output_file + '.txt'
     if options.sidecar == options.input_file or options.sidecar == options.output_file:
@@ -195,16 +205,6 @@ def check_options_ocr_behavior(options: Namespace) -> None:
         options.pages = _pages_from_ranges(options.pages)
 
 
-def check_options_advanced(options: Namespace) -> None:
-    if options.pdfa_image_compression != 'auto' and not options.output_type.startswith(
-        'pdfa'
-    ):
-        log.warning(
-            "--pdfa-image-compression argument only applies when "
-            "--output-type is one of 'pdfa', 'pdfa-1', or 'pdfa-2'"
-        )
-
-
 def check_options_metadata(options: Namespace) -> None:
     docinfo = [options.title, options.author, options.keywords, options.subject]
     for s in (m for m in docinfo if m):
@@ -231,7 +231,6 @@ def _check_plugin_invariant_options(options: Namespace) -> None:
     check_options_sidecar(options)
     check_options_preprocessing(options)
     check_options_ocr_behavior(options)
-    check_options_advanced(options)
     check_options_pillow(options)
 
 
@@ -276,7 +275,8 @@ def create_input_file(options: Namespace, work_folder: Path) -> tuple[Path, str]
                     "permissions correctly.\n"
                     "You may find it easier to use stdin/stdout:"
                     "\n"
-                    "\tdocker run -i --rm jbarlow83/ocrmypdf - - <input.pdf >output.pdf\n"
+                    "\tdocker run -i --rm jbarlow83/ocrmypdf - - <input.pdf >output.pdf"
+                    "\n"
                 )
             raise InputFileError(msg) from e
 
@@ -333,7 +333,7 @@ def report_output_file_size(
     for arg in image_preproc:
         if getattr(options, arg, False):
             reasons.append(
-                f"The argument --{arg.replace('_', '-')} was issued, causing transcoding."
+                f"--{arg.replace('_', '-')} was issued, causing transcoding."
             )
 
     reasons.extend(optimize_messages)
