@@ -8,10 +8,14 @@ from __future__ import annotations
 import logging
 import os
 
+from PIL import Image
+
 from ocrmypdf import hookimpl
 from ocrmypdf._exec import tesseract
+from ocrmypdf._jobcontext import PageContext
 from ocrmypdf.cli import numeric, str_to_int
 from ocrmypdf.helpers import clamp
+from ocrmypdf.imageops import calculate_downsample, downsample_image
 from ocrmypdf.pluginspec import OcrEngine
 from ocrmypdf.subprocess import check_external_program
 
@@ -26,7 +30,7 @@ def add_options(parser):
         action='append',
         metavar='CFG',
         default=[],
-        help="Additional Tesseract configuration files -- see documentation",
+        help="Additional Tesseract configuration files -- see documentation.",
     )
     tess.add_argument(
         '--tesseract-pagesegmode',
@@ -34,7 +38,7 @@ def add_options(parser):
         type=int,
         metavar='PSM',
         choices=range(0, 14),
-        help="Set Tesseract page segmentation mode (see tesseract --help)",
+        help="Set Tesseract page segmentation mode (see tesseract --help).",
     )
     tess.add_argument(
         '--tesseract-oem',
@@ -58,7 +62,7 @@ def add_options(parser):
         metavar='METHOD',
         help=(
             "Set Tesseract 5.0+ input image thresholding mode. This may improve OCR "
-            "results on low quality images or those that contain high constrast color. "
+            "results on low quality images or those that contain high contrast color. "
             "legacy-otsu is the Tesseract default; adaptive-otsu is an improved Otsu "
             "algorithm with improved sort for background color changes; sauvola is "
             "based on local standard deviation."
@@ -69,8 +73,51 @@ def add_options(parser):
         default=180.0,
         type=numeric(float, 0),
         metavar='SECONDS',
-        help='Give up on OCR after the timeout, but copy the preprocessed page '
-        'into the final output',
+        help=(
+            "Give up on OCR after the timeout, but copy the preprocessed page "
+            "into the final output. This timeout is only used when using Tesseract "
+            "for OCR. When Tesseract is used for other operations such as "
+            "deskewing and orientation, the timeout is controlled by "
+            "--tesseract-non-ocr-timeout."
+        ),
+    )
+    tess.add_argument(
+        '--tesseract-non-ocr-timeout',
+        default=180.0,
+        type=numeric(float, 0),
+        metavar='SECONDS',
+        help=(
+            "Give up on non-OCR operations such as deskewing and orientation "
+            "after timeout. This is a separate timeout from --tesseract-timeout "
+            "because these operations are not as expensive as OCR."
+        ),
+    )
+    tess.add_argument(
+        '--tesseract-downsample-large-images',
+        action='store_true',
+        help=(
+            "Downsample large images before OCR. Tesseract has an upper limit on the "
+            "size images it will support. If this argument is given, OCRmyPDF will "
+            "downsample large images to fit Tesseract. This may reduce OCR quality, "
+            "on large images the most desirable text is usually larger. If this "
+            "parameter is not supplied, Tesseract will error out and produce no OCR "
+            "on the page in question. This argument should be used with a high value "
+            "of --tesseract-timeout to ensure Tesseract has enough to time."
+        ),
+    )
+    tess.add_argument(
+        '--tesseract-downsample-above',
+        action='store',
+        type=numeric(int, 100, 32767),
+        default=32767,
+        help=(
+            "Downsample images larger than this size pixel size in either dimension "
+            "before OCR. --tesseract-downsample-large-images downsamples only when "
+            "an image exceeds Tesseract's internal limits. This argument causes "
+            "downsampling to occur when an image exceeds the given size. This may "
+            "reduce OCR quality, but on large images the most desirable text is "
+            "usually larger."
+        ),
     )
     tess.add_argument(
         '--user-words',
@@ -93,7 +140,7 @@ def check_options(options):
         program='tesseract',
         package={'linux': 'tesseract-ocr'},
         version_checker=tesseract.version,
-        need_version='4.1.1',  # Ubuntu 20.04 version
+        need_version='4.1.1',  # Ubuntu 22.04 version (also 20.04)
         version_parser=tesseract.TesseractVersion,
     )
 
@@ -131,13 +178,41 @@ def validate(pdfinfo, options):
         tess_threads = int(os.environ['OMP_THREAD_LIMIT'])
     log.debug("Using Tesseract OpenMP thread limit %d", tess_threads)
 
+    if (
+        options.tesseract_downsample_above != 32767
+        and not options.tesseract_downsample_large_images
+    ):
+        log.warning(
+            "The --tesseract-downsample-above argument will have no effect unless "
+            "--tesseract-downsample-large-images is also given."
+        )
+
+
+@hookimpl
+def filter_ocr_image(page: PageContext, image: Image.Image) -> Image.Image:
+    """Filter the image before OCR.
+
+    Tesseract cannot handle images with more than 32767 pixels in either axis,
+    or more than 2**31 bytes. This function resizes the image to fit within
+    those limits.
+    """
+    threshold = min(page.options.tesseract_downsample_above, 32767)
+
+    options = page.options
+    if options.tesseract_downsample_large_images:
+        size = calculate_downsample(
+            image, max_size=(threshold, threshold), max_bytes=(2**31) - 1
+        )
+        image = downsample_image(image, size)
+    return image
+
 
 class TesseractOcrEngine(OcrEngine):
     """Implements OCR with Tesseract."""
 
     @staticmethod
     def version():
-        return tesseract.version()
+        return str(tesseract.version())
 
     @staticmethod
     def creator_tag(options):
@@ -156,7 +231,7 @@ class TesseractOcrEngine(OcrEngine):
         return tesseract.get_orientation(
             input_file,
             engine_mode=options.tesseract_oem,
-            timeout=options.tesseract_timeout,
+            timeout=options.tesseract_non_ocr_timeout,
         )
 
     @staticmethod
@@ -165,7 +240,7 @@ class TesseractOcrEngine(OcrEngine):
             input_file,
             languages=options.languages,
             engine_mode=options.tesseract_oem,
-            timeout=options.tesseract_timeout,
+            timeout=options.tesseract_non_ocr_timeout,
         )
 
     @staticmethod
