@@ -6,7 +6,8 @@
 from __future__ import annotations
 
 import argparse
-from typing import Any, Callable, Mapping, TypeVar
+from collections.abc import Mapping
+from typing import Any, Callable, TypeVar
 
 from ocrmypdf._version import PROGRAM_NAME as _PROGRAM_NAME
 from ocrmypdf._version import __version__ as _VERSION
@@ -15,7 +16,11 @@ T = TypeVar('T', int, float)
 
 
 def numeric(basetype: Callable[[Any], T], min_: T | None = None, max_: T | None = None):
-    """Validator for numeric params"""
+    """Validator for numeric command line parameters.
+
+    Stipulates that the value must be of type basetype (typically int or float), and
+    optionally, within the range [min_, max_].
+    """
     min_ = basetype(min_) if min_ is not None else None
     max_ = basetype(max_) if max_ is not None else None
 
@@ -46,7 +51,7 @@ def str_to_int(mapping: Mapping[str, int]):
 
 
 class ArgumentParser(argparse.ArgumentParser):
-    """Override parser's default behavior of calling sys.exit()
+    """Override parser's default behavior of calling sys.exit().
 
     https://stackoverflow.com/questions/5943249/python-argparse-and-controlling-overriding-the-exit-status-code
 
@@ -57,13 +62,21 @@ class ArgumentParser(argparse.ArgumentParser):
     """
 
     def __init__(self, *args, **kwargs):
+        """Initialize the parser."""
         super().__init__(*args, **kwargs)
         self._api_mode = False
 
     def enable_api_mode(self):
+        """Enable API mode.
+
+        When set, the parser will not call sys.exit() on error. OCRmyPDF was originally
+        a command line program, but now it has an API. The API works by synthesizing
+        command line arguments.
+        """
         self._api_mode = True
 
     def error(self, message):
+        """Override the default argparse error behavior."""
         if not self._api_mode:
             super().error(message)
             return
@@ -74,19 +87,22 @@ class LanguageSetAction(argparse.Action):
     """Manages a list of languages."""
 
     def __init__(self, option_strings, dest, default=None, **kwargs):
+        """Initialize the action."""
         if default is None:
-            default = set()
+            default = list()
         super().__init__(option_strings, dest, default=default, **kwargs)
 
     def __call__(self, parser, namespace, values, option_string=None):
+        """Add a language to the set."""
         dest = getattr(namespace, self.dest)
         if '+' in values:
-            dest.update(lang for lang in values.split('+'))
+            [dest.append(lang) for lang in values.split('+')]
         else:
-            dest.add(values)
+            dest.append(values)
 
 
 def get_parser():
+    """Get the main CLI parser."""
     parser = ArgumentParser(
         prog=_PROGRAM_NAME,
         allow_abbrev=True,
@@ -166,7 +182,9 @@ Online documentation is located at:
         '--image-dpi',
         metavar='DPI',
         type=int,
-        help="For input image instead of PDF, use this DPI instead of file's.",
+        help="When the input file is an image, not a PDF, use this DPI instead "
+        "of the DPI claimed by the input file. If the input does not claim a "
+        "sensible DPI, this option will be required.",
     )
     parser.add_argument(
         '--output-type',
@@ -348,6 +366,13 @@ Online documentation is located at:
         help="Skip OCR on pages larger than the specified amount of megapixels, "
         "but include skipped pages in final output",
     )
+    ocrsettings.add_argument(
+        '--invalidate-digital-signatures',
+        action='store_true',
+        help="Normally, OCRmyPDF will refuse to OCR a PDF that has a digital "
+        "signature. This option allows OCR to proceed, but the digital signature "
+        "will be invalidated.",
+    )
 
     advanced = parser.add_argument_group(
         "Advanced", "Advanced options to control OCRmyPDF"
@@ -384,19 +409,6 @@ Online documentation is located at:
         help="Only rotate pages when confidence is above this value (arbitrary "
         "units reported by tesseract)",
     )
-    advanced.add_argument(
-        '--pdfa-image-compression',
-        choices=['auto', 'jpeg', 'lossless'],
-        default='auto',
-        help="Specify how to compress images in the output PDF/A. 'auto' lets "
-        "OCRmyPDF decide.  'jpeg' changes all grayscale and color images to "
-        "JPEG compression.  'lossless' uses PNG-style lossless compression "
-        "for all images.  Monochrome images are always compressed using a "
-        "lossless codec.  Compression settings "
-        "are applied to all pages, including those for which OCR was "
-        "skipped.  Not supported for --output-type=pdf ; that setting "
-        "preserves the original compression of all images.",
-    )
     advanced.add_argument(
         '--fast-web-view',
         type=numeric(float, 0),
@@ -409,6 +421,14 @@ Online documentation is located at:
         "which do not benefit. If the threshold is 0 it will be apply to all files. "
         "Set the threshold very high to disable.",
     )
+    advanced.add_argument(
+        '--continue-on-soft-render-error',
+        action='store_true',
+        help="Continue processing pages after a recoverable PDF rendering error. "
+        "A recoverable error is one that does not prevent the page from being "
+        "rendered, but may result in visual differences compared to the input "
+        "file. Missing fonts are a typical source of these errors.",
+    )
     advanced.add_argument(
         '--plugin',
         dest='plugins',
