@@ -10,30 +10,29 @@ import multiprocessing
 import os
 import shutil
 import warnings
-from collections.abc import Iterable
+from collections.abc import Iterable, Sequence
 from contextlib import suppress
+from decimal import Decimal
 from io import StringIO
 from math import isclose, isfinite
 from pathlib import Path
-from typing import Any, Generic, Sequence, SupportsFloat, SupportsRound, TypeVar
+from statistics import harmonic_mean
+from typing import (
+    Any,
+    Callable,
+    Generic,
+    TypeVar,
+)
 
 import img2pdf
 import pikepdf
-from packaging.version import Version
 
 log = logging.getLogger(__name__)
 
-if Version(img2pdf.__version__) < Version('0.4.0'):
-    IMG2PDF_KWARGS = dict(without_pdfw=True)
-elif Version(img2pdf.__version__) < Version('0.4.3'):
-    IMG2PDF_KWARGS = dict(engine=img2pdf.Engine.pikepdf)
-else:
-    IMG2PDF_KWARGS = dict(
-        engine=img2pdf.Engine.pikepdf, rotation=img2pdf.Rotation.ifvalid
-    )
+IMG2PDF_KWARGS = dict(engine=img2pdf.Engine.pikepdf, rotation=img2pdf.Rotation.ifvalid)
 
 
-T = TypeVar('T', bound=SupportsRound[Any])
+T = TypeVar('T', float, int, Decimal)
 
 
 class Resolution(Generic[T]):
@@ -49,6 +48,7 @@ class Resolution(Generic[T]):
     __slots__ = ('x', 'y')
 
     def __init__(self, x: T, y: T):
+        """Construct a Resolution object."""
         self.x = x
         self.y = y
 
@@ -57,9 +57,11 @@ class Resolution(Generic[T]):
     CONVERSION_ERROR = 0.002
 
     def round(self, ndigits: int) -> Resolution:
+        """Round to ndigits after the decimal point."""
         return Resolution(round(self.x, ndigits), round(self.y, ndigits))
 
     def to_int(self) -> Resolution[int]:
+        """Round to nearest integer."""
         return Resolution(int(round(self.x)), int(round(self.y)))
 
     @classmethod
@@ -68,38 +70,65 @@ class Resolution(Generic[T]):
 
     @property
     def is_square(self) -> bool:
+        """True if the resolution is square (x == y)."""
         return self._isclose(self.x, self.y)
 
     @property
     def is_finite(self) -> bool:
-        if isinstance(self.x, SupportsFloat) and isinstance(self.y, SupportsFloat):
-            return isfinite(self.x) and isfinite(self.y)
-        return True
+        """True if both x and y are finite numbers."""
+        return isfinite(self.x) and isfinite(self.y)
+
+    def to_scalar(self) -> float:
+        """Return the harmonic mean of x and y as a 1D approximation.
+
+        In most cases, Resolution is 2D, but typically it is "square" (x == y) and
+        can be approximated as a single number. When not square, the harmonic mean
+        is used to approximate the 2D resolution as a single number.
+        """
+        return harmonic_mean([float(self.x), float(self.y)])
+
+    def _take_minmax(
+        self, vals: Iterable[Any], yvals: Iterable[Any] | None, cmp: Callable
+    ) -> Resolution:
+        """Return a new Resolution object with the maximum resolution of inputs."""
+        if yvals is not None:
+            return Resolution(cmp(self.x, *vals), cmp(self.y, *yvals))
+        cmp_x, cmp_y = self.x, self.y
+        for x, y in vals:
+            cmp_x = cmp(x, cmp_x)
+            cmp_y = cmp(y, cmp_y)
+        return Resolution(cmp_x, cmp_y)
 
     def take_max(
         self, vals: Iterable[Any], yvals: Iterable[Any] | None = None
     ) -> Resolution:
-        if yvals is not None:
-            return Resolution(max(self.x, *vals), max(self.y, *yvals))
-        max_x, max_y = self.x, self.y
-        for x, y in vals:
-            max_x = max(x, max_x)
-            max_y = max(y, max_y)
-        return Resolution(max_x, max_y)
+        """Return a new Resolution object with the maximum resolution of inputs."""
+        return self._take_minmax(vals, yvals, max)
+
+    def take_min(
+        self, vals: Iterable[Any], yvals: Iterable[Any] | None = None
+    ) -> Resolution:
+        """Return a new Resolution object with the minimum resolution of inputs."""
+        return self._take_minmax(vals, yvals, min)
 
     def flip_axis(self) -> Resolution[T]:
+        """Return a new Resolution object with x and y swapped."""
         return Resolution(self.y, self.x)
 
     def __getitem__(self, idx: int | slice) -> T:
+        """Support [0] and [1] indexing."""
         return (self.x, self.y)[idx]
 
     def __str__(self):
-        return f"{self.x:f}x{self.y:f}"
+        """Return a string representation of the resolution."""
+        return f"{self.x:f}×{self.y:f}"
 
     def __repr__(self):  # pragma: no cover
-        return f"Resolution({self.x}x{self.y} dpi)"
+        """Return a repr() of the resolution."""
+        return f"Resolution({self.x!r}, {self.y!r})"
 
     def __eq__(self, other):
+        """Return True if the resolution is equal to another resolution."""
         if isinstance(other, tuple) and len(other) == 2:
             other = Resolution(*other)
         if not isinstance(other, Resolution):
@@ -108,10 +137,10 @@ class Resolution(Generic[T]):
 
 
 class NeverRaise(Exception):
-    """An exception that is never raised"""
+    """An exception that is never raised."""
 
 
-def safe_symlink(input_file: os.PathLike, soft_link_name: os.PathLike):
+def safe_symlink(input_file: os.PathLike, soft_link_name: os.PathLike) -> None:
     """Create a symbolic link at ``soft_link_name``, which references ``input_file``.
 
     Think of this as copying ``input_file`` to ``soft_link_name`` with less overhead.
@@ -126,7 +155,7 @@ def safe_symlink(input_file: os.PathLike, soft_link_name: os.PathLike):
     # Guard against soft linking to oneself
     if input_file == soft_link_name:
         log.warning(
-            "No symbolic link created. You are using  the original data directory "
+            "No symbolic link created. You are using the original data directory "
             "as the working directory."
         )
         return
@@ -152,7 +181,11 @@ def safe_symlink(input_file: os.PathLike, soft_link_name: os.PathLike):
     os.symlink(os.path.abspath(input_file), soft_link_name)
 
 
-def samefile(file1: os.PathLike, file2: os.PathLike):
+def samefile(file1: os.PathLike, file2: os.PathLike) -> bool:
+    """Return True if two files are the same file.
+
+    Attempts to account for different relative paths to the same file.
+    """
     if os.name == 'nt':
         return file1 == file2
     else:
@@ -170,7 +203,7 @@ def monotonic(seq: Sequence) -> bool:
 
 
 def page_number(input_file: os.PathLike) -> int:
-    """Get one-based page number implied by filename (000002.pdf -> 2)"""
+    """Get one-based page number implied by filename (000002.pdf -> 2)."""
     return int(os.path.basename(os.fspath(input_file))[0:6])
 
 
@@ -199,7 +232,7 @@ def is_file_writable(test_file: os.PathLike) -> bool:
             p = p.resolve(strict=False)
 
         # p.is_file() throws an exception in some cases
-        if p.exists() and p.is_file():
+        if p.exists() and (p.is_file() or p.samefile(os.devnull)):
             return os.access(
                 os.fspath(p),
                 os.W_OK,
@@ -267,26 +300,29 @@ def check_pdf(input_file: Path) -> bool:
             return False
 
 
-def clamp(n, smallest, largest):  # mypy doesn't understand types for this
+def clamp(n: T, smallest: T, largest: T) -> T:
     """Clamps the value of ``n`` to between ``smallest`` and ``largest``."""
     return max(smallest, min(n, largest))
 
 
-def remove_all_log_handlers(logger):
-    "Remove all log handlers, usually used in a child process."
+def remove_all_log_handlers(logger: logging.Logger) -> None:
+    """Remove all log handlers, usually used in a child process.
+
+    The child process inherits the log handlers from the parent process when
+    a fork occurs. Typically we want to remove all log handlers in the child
+    process so that the child process can set up a single queue handler to
+    forward log messages to the parent process.
+    """
     for handler in logger.handlers[:]:
         logger.removeHandler(handler)
         handler.close()  # To ensure handlers with opened resources are released
 
 
-def pikepdf_enable_mmap():
-    # try:
-    #     if pikepdf._qpdf.set_access_default_mmap(True):
-    #         log.debug("pikepdf mmap enabled")
-    # except AttributeError:
-    #     log.debug("pikepdf mmap not available")
-    # We found a race condition probably related to pybind issue #2252 that can
-    # cause a crash. For now, disable pikepdf mmap to be on the safe side.
-    # Fix is not in pybind11 2.6.0
-    # log.debug("pikepdf mmap disabled")
-    return
+def pikepdf_enable_mmap() -> None:
+    """Enable pikepdf mmap."""
+    try:
+        if pikepdf._core.set_access_default_mmap(True):
+            log.debug("pikepdf mmap enabled")
+    except AttributeError:
+        log.debug("pikepdf mmap not available")
+    log.debug("pikepdf mmap disabled")
