@@ -11,9 +11,10 @@ import sys
 import tempfile
 import threading
 from collections import defaultdict
+from collections.abc import Iterator, MutableSet, Sequence
 from os import fspath
 from pathlib import Path
-from typing import Callable, Iterator, MutableSet, NamedTuple, NewType, Sequence
+from typing import Callable, NamedTuple, NewType
 from zlib import compress
 
 import img2pdf
@@ -53,21 +54,25 @@ class XrefExt(NamedTuple):
 
 
 def img_name(root: Path, xref: Xref, ext: str) -> Path:
+    """Return the name of an image file for a given xref and extension."""
     return root / f'{xref:08d}{ext}'
 
 
 def png_name(root: Path, xref: Xref) -> Path:
+    """Return the name of a PNG file for a given xref."""
     return img_name(root, xref, '.png')
 
 
 def jpg_name(root: Path, xref: Xref) -> Path:
+    """Return the name of a JPEG file for a given xref."""
     return img_name(root, xref, '.jpg')
 
 
 def extract_image_filter(
-    pike: Pdf, root: Path, image: Stream, xref: Xref
+    pdf: Pdf, root: Path, image: Stream, xref: Xref
 ) -> tuple[PdfImage, tuple[Name, Object]] | None:
-    del pike  # unused args
+    """Determine if an image is extractable."""
+    del pdf  # unused args
     del root
 
     if image.Subtype != Name.Image:
@@ -122,11 +127,12 @@ def extract_image_filter(
 
 
 def extract_image_jbig2(
-    *, pike: Pdf, root: Path, image: Stream, xref: Xref, options
+    *, pdf: Pdf, root: Path, image: Stream, xref: Xref, options
 ) -> XrefExt | None:
+    """Extract an image, saving it as a JBIG2 file."""
     del options  # unused arg
 
-    result = extract_image_filter(pike, root, image, xref)
+    result = extract_image_filter(pdf, root, image, xref)
     if result is None:
         return None
     pim, filtdp = result
@@ -163,9 +169,10 @@ def extract_image_jbig2(
 
 
 def extract_image_generic(
-    *, pike: Pdf, root: Path, image: Stream, xref: Xref, options
+    *, pdf: Pdf, root: Path, image: Stream, xref: Xref, options
 ) -> XrefExt | None:
-    result = extract_image_filter(pike, root, image, xref)
+    """Generic image extraction."""
+    result = extract_image_filter(pdf, root, image, xref)
     if result is None:
         return None
     pim, filtdp = result
@@ -224,13 +231,72 @@ def extract_image_generic(
     return None
 
 
+def _find_image_xrefs_container(
+    pdf: Pdf,
+    container: Object,
+    pageno: int,
+    include_xrefs: MutableSet[Xref],
+    exclude_xrefs: MutableSet[Xref],
+    pageno_for_xref: dict[Xref, int],
+    depth: int = 0,
+):
+    """Find all image XRefs or Form XObject and add to the include/exclude sets."""
+    if depth > 10:
+        log.warning("Recursion depth exceeded in _find_image_xrefs_page")
+        return
+    try:
+        xobjs = container.Resources.XObject
+    except AttributeError:
+        return
+    for _imname, image in dict(xobjs).items():
+        if image.objgen[1] != 0:
+            continue  # Ignore images in an incremental PDF
+        if Name.Subtype in image and image.Subtype == Name.Form:
+            # Recurse into Form XObjects
+            log.debug(f"Recursing into Form XObject {_imname} in page {pageno}")
+            _find_image_xrefs_container(
+                pdf,
+                image,
+                pageno,
+                include_xrefs,
+                exclude_xrefs,
+                pageno_for_xref,
+                depth + 1,
+            )
+            continue
+        xref = Xref(image.objgen[0])
+        if Name.SMask in image:
+            # Ignore soft masks
+            smask_xref = Xref(image.SMask.objgen[0])
+            exclude_xrefs.add(smask_xref)
+            log.debug(f"xref {smask_xref}: skipping image because it is an SMask")
+        include_xrefs.add(xref)
+        log.debug(f"xref {xref}: treating as an optimization candidate")
+        if xref not in pageno_for_xref:
+            pageno_for_xref[xref] = pageno
+
+
+def _find_image_xrefs(pdf: Pdf):
+    include_xrefs: MutableSet[Xref] = set()
+    exclude_xrefs: MutableSet[Xref] = set()
+    pageno_for_xref: dict[Xref, int] = {}
+
+    for pageno, page in enumerate(pdf.pages):
+        _find_image_xrefs_container(
+            pdf, page, pageno, include_xrefs, exclude_xrefs, pageno_for_xref
+        )
+
+    working_xrefs = include_xrefs - exclude_xrefs
+    return working_xrefs, pageno_for_xref
+
+
 def extract_images(
-    pike: Pdf,
+    pdf: Pdf,
     root: Path,
     options,
     extract_fn: Callable[..., XrefExt | None],
 ) -> Iterator[tuple[int, XrefExt]]:
-    """Extract image using extract_fn
+    """Extract image using extract_fn.
 
     Enumerate images on each page, lookup their xref/ID number in the PDF.
     Exclude images that are soft masks (i.e. alpha transparency related).
@@ -244,36 +310,13 @@ def extract_images(
     it does a tuple should be returned: (xref, ext) where .ext is the file
     extension. extract_fn must also extract the file it finds interesting.
     """
-
-    include_xrefs: MutableSet[Xref] = set()
-    exclude_xrefs: MutableSet[Xref] = set()
-    pageno_for_xref = {}
     errors = 0
-    for pageno, page in enumerate(pike.pages):
-        try:
-            xobjs = page.Resources.XObject
-        except AttributeError:
-            continue
-        for _imname, image in dict(xobjs).items():
-            if image.objgen[1] != 0:
-                continue  # Ignore images in an incremental PDF
-            xref = Xref(image.objgen[0])
-            if Name.SMask in image:
-                # Ignore soft masks
-                smask_xref = Xref(image.SMask.objgen[0])
-                exclude_xrefs.add(smask_xref)
-                log.debug(f"xref {smask_xref}: skipping image because it is an SMask")
-            include_xrefs.add(xref)
-            log.debug(f"xref {xref}: treating as an optimization candidate")
-            if xref not in pageno_for_xref:
-                pageno_for_xref[xref] = pageno
-
-    working_xrefs = include_xrefs - exclude_xrefs
+    working_xrefs, pageno_for_xref = _find_image_xrefs(pdf)
     for xref in working_xrefs:
-        image = pike.get_object((xref, 0))
+        image = pdf.get_object((xref, 0))
         try:
             result = extract_fn(
-                pike=pike, root=root, image=image, xref=xref, options=options
+                pdf=pdf, root=root, image=image, xref=xref, options=options
             )
         except Exception:  # pylint: disable=broad-except
             log.exception(
@@ -287,13 +330,12 @@ def extract_images(
 
 
 def extract_images_generic(
-    pike: Pdf, root: Path, options
+    pdf: Pdf, root: Path, options
 ) -> tuple[list[Xref], list[Xref]]:
-    """Extract any >=2bpp image we think we can improve"""
-
+    """Extract any >=2bpp image we think we can improve."""
     jpegs = []
     pngs = []
-    for _, xref_ext in extract_images(pike, root, options, extract_image_generic):
+    for _, xref_ext in extract_images(pdf, root, options, extract_image_generic):
         log.debug('%s', xref_ext)
         if xref_ext.ext == '.png':
             pngs.append(xref_ext.xref)
@@ -303,11 +345,10 @@ def extract_images_generic(
     return jpegs, pngs
 
 
-def extract_images_jbig2(pike: Pdf, root: Path, options) -> dict[int, list[XrefExt]]:
-    """Extract any bitonal image that we think we can improve as JBIG2"""
-
+def extract_images_jbig2(pdf: Pdf, root: Path, options) -> dict[int, list[XrefExt]]:
+    """Extract any bitonal image that we think we can improve as JBIG2."""
     jbig2_groups = defaultdict(list)
-    for pageno, xref_ext in extract_images(pike, root, options, extract_image_jbig2):
+    for pageno, xref_ext in extract_images(pdf, root, options, extract_image_jbig2):
         group = pageno // options.jbig2_page_group_size
         jbig2_groups[group].append(xref_ext)
 
@@ -318,7 +359,7 @@ def extract_images_jbig2(pike: Pdf, root: Path, options) -> dict[int, list[XrefE
 def _produce_jbig2_images(
     jbig2_groups: dict[int, list[XrefExt]], root: Path, options, executor: Executor
 ) -> None:
-    """Produce JBIG2 images from their groups"""
+    """Produce JBIG2 images from their groups."""
 
     def jbig2_group_args(root: Path, groups: dict[int, list[XrefExt]]):
         for group, xref_exts in groups.items():
@@ -327,6 +368,7 @@ def _produce_jbig2_images(
                 fspath(root),  # =cwd
                 (img_name(root, xref, ext) for xref, ext in xref_exts),  # =infiles
                 prefix,  # =out_prefix
+                options.jbig2_threshold,
             )
 
     def jbig2_single_args(root, groups: dict[int, list[XrefExt]]):
@@ -339,6 +381,7 @@ def _produce_jbig2_images(
                     fspath(root),
                     img_name(root, xref, ext),
                     root / f'{prefix}.{n:04d}',
+                    options.jbig2_threshold,
                 )
 
     if options.jbig2_page_group_size > 1:
@@ -363,7 +406,7 @@ def _produce_jbig2_images(
 
 
 def convert_to_jbig2(
-    pike: Pdf,
+    pdf: Pdf,
     jbig2_groups: dict[int, list[XrefExt]],
     root: Path,
     options,
@@ -390,7 +433,7 @@ def convert_to_jbig2(
         jbig2_symfile = root / (prefix + '.sym')
         if jbig2_symfile.exists():
             jbig2_globals_data = jbig2_symfile.read_bytes()
-            jbig2_globals = Stream(pike, jbig2_globals_data)
+            jbig2_globals = Stream(pdf, jbig2_globals_data)
             jbig2_globals_dict = Dictionary(JBIG2Globals=jbig2_globals)
         elif options.jbig2_page_group_size == 1:
             jbig2_globals_dict = None
@@ -401,7 +444,7 @@ def convert_to_jbig2(
             xref, _ = xref_ext
             jbig2_im_file = root / (prefix + f'.{n:04d}')
             jbig2_im_data = jbig2_im_file.read_bytes()
-            im_obj = pike.get_object(xref, 0)
+            im_obj = pdf.get_object(xref, 0)
             im_obj.write(
                 jbig2_im_data, filter=Name.JBIG2Decode, decode_parms=jbig2_globals_dict
             )
@@ -421,8 +464,10 @@ def _optimize_jpeg(args: tuple[Xref, Path, Path, int]) -> tuple[Xref, Path | Non
 
 
 def transcode_jpegs(
-    pike: Pdf, jpegs: Sequence[Xref], root: Path, options, executor: Executor
+    pdf: Pdf, jpegs: Sequence[Xref], root: Path, options, executor: Executor
 ) -> None:
+    """Optimize JPEGs according to optimization settings."""
+
     def jpeg_args() -> Iterator[tuple[Xref, Path, Path, int]]:
         for xref in jpegs:
             in_jpg = jpg_name(root, xref)
@@ -433,7 +478,7 @@ def transcode_jpegs(
         xref, opt_jpg = result
         if opt_jpg:
             compdata = opt_jpg.read_bytes()  # JPEG can inserted into PDF as is
-            im_obj = pike.get_object(xref, 0)
+            im_obj = pdf.get_object(xref, 0)
             im_obj.write(compdata, filter=Name.DCTDecode)
         pbar.update()
 
@@ -453,9 +498,9 @@ def transcode_jpegs(
 
 
 def _find_deflatable_jpeg(
-    *, pike: Pdf, root: Path, image: Stream, xref: Xref, options
+    *, pdf: Pdf, root: Path, image: Stream, xref: Xref, options
 ) -> XrefExt | None:
-    result = extract_image_filter(pike, root, image, xref)
+    result = extract_image_filter(pdf, root, image, xref)
     if result is None:
         return None
     _pim, filtdp = result
@@ -467,9 +512,9 @@ def _find_deflatable_jpeg(
 
 
 def _deflate_jpeg(args: tuple[Pdf, threading.Lock, Xref, int]) -> tuple[Xref, bytes]:
-    pike, lock, xref, complevel = args
+    pdf, lock, xref, complevel = args
     with lock:
-        xobj = pike.get_object(xref, 0)
+        xobj = pdf.get_object(xref, 0)
         try:
             data = xobj.read_raw_bytes()
         except PdfError:
@@ -480,9 +525,15 @@ def _deflate_jpeg(args: tuple[Pdf, threading.Lock, Xref, int]) -> tuple[Xref, by
     return xref, compdata
 
 
-def deflate_jpegs(pike: Pdf, root: Path, options, executor: Executor) -> None:
+def deflate_jpegs(pdf: Pdf, root: Path, options, executor: Executor) -> None:
+    """Apply FlateDecode to JPEGs.
+
+    This is a lossless compression method that is supported by all PDF viewers,
+    and generally results in a smaller file size compared to straight DCTDecode
+    images.
+    """
     jpegs = []
-    for _pageno, xref_ext in extract_images(pike, root, options, _find_deflatable_jpeg):
+    for _pageno, xref_ext in extract_images(pdf, root, options, _find_deflatable_jpeg):
         xref = xref_ext.xref
         log.debug(f'xref {xref}: marking this JPEG as deflatable')
         jpegs.append(xref)
@@ -494,13 +545,13 @@ def deflate_jpegs(pike: Pdf, root: Path, options, executor: Executor) -> None:
 
     def deflate_args() -> Iterator:
         for xref in jpegs:
-            yield pike, lock, xref, complevel
+            yield pdf, lock, xref, complevel
 
     def finish(result, pbar):
         xref, compdata = result
         if len(compdata) > 0:
             with lock:
-                xobj = pike.get_object(xref, 0)
+                xobj = pdf.get_object(xref, 0)
                 xobj.write(compdata, filter=[Name.FlateDecode, Name.DCTDecode])
         pbar.update()
 
@@ -519,16 +570,16 @@ def deflate_jpegs(pike: Pdf, root: Path, options, executor: Executor) -> None:
     )
 
 
-def _transcode_png(pike: Pdf, filename: Path, xref: Xref) -> bool:
+def _transcode_png(pdf: Pdf, filename: Path, xref: Xref) -> bool:
     output = filename.with_suffix('.png.pdf')
     with output.open('wb') as f:
         img2pdf.convert(fspath(filename), outputstream=f, **IMG2PDF_KWARGS)
 
     with Pdf.open(output) as pdf_image:
         foreign_image = next(iter(pdf_image.pages[0].images.values()))
-        local_image = pike.copy_foreign(foreign_image)
+        local_image = pdf.copy_foreign(foreign_image)
 
-        im_obj = pike.get_object(xref, 0)
+        im_obj = pdf.get_object(xref, 0)
         im_obj.write(
             local_image.read_raw_bytes(),
             filter=local_image.Filter,
@@ -561,13 +612,14 @@ def _transcode_png(pike: Pdf, filename: Path, xref: Xref) -> bool:
 
 
 def transcode_pngs(
-    pike: Pdf,
+    pdf: Pdf,
     images: Sequence[Xref],
     image_name_fn: Callable[[Path, Xref], Path],
     root: Path,
     options,
     executor,
 ) -> None:
+    """Apply lossy transcoding to PNGs."""
     modified: MutableSet[Xref] = set()
     if options.optimize >= 2:
         png_quality = (
@@ -601,7 +653,7 @@ def transcode_pngs(
 
     for xref in modified:
         filename = png_name(root, xref)
-        _transcode_png(pike, filename, xref)
+        _transcode_png(pdf, filename, xref)
 
 
 DEFAULT_EXECUTOR = SerialExecutor()
@@ -614,6 +666,7 @@ def optimize(
     save_settings,
     executor: Executor = DEFAULT_EXECUTOR,
 ) -> Path:
+    """Optimize images in a PDF file."""
     options = context.options
     if options.optimize == 0:
         safe_symlink(input_file, output_file)
@@ -626,24 +679,24 @@ def optimize(
     if options.jbig2_page_group_size == 0:
         options.jbig2_page_group_size = 10 if options.jbig2_lossy else 1
 
-    with Pdf.open(input_file) as pike:
+    with Pdf.open(input_file) as pdf:
         root = output_file.parent / 'images'
         root.mkdir(exist_ok=True)
 
-        jpegs, pngs = extract_images_generic(pike, root, options)
-        transcode_jpegs(pike, jpegs, root, options, executor)
-        deflate_jpegs(pike, root, options, executor)
+        jpegs, pngs = extract_images_generic(pdf, root, options)
+        transcode_jpegs(pdf, jpegs, root, options, executor)
+        deflate_jpegs(pdf, root, options, executor)
         # if options.optimize >= 2:
         # Try pngifying the jpegs
-        #    transcode_pngs(pike, jpegs, jpg_name, root, options)
-        transcode_pngs(pike, pngs, png_name, root, options, executor)
+        #    transcode_pngs(pdf, jpegs, jpg_name, root, options)
+        transcode_pngs(pdf, pngs, png_name, root, options, executor)
 
-        jbig2_groups = extract_images_jbig2(pike, root, options)
-        convert_to_jbig2(pike, jbig2_groups, root, options, executor)
+        jbig2_groups = extract_images_jbig2(pdf, root, options)
+        convert_to_jbig2(pdf, jbig2_groups, root, options, executor)
 
         target_file = output_file.with_suffix('.opt.pdf')
-        pike.remove_unreferenced_resources()
-        pike.save(target_file, **save_settings)
+        pdf.remove_unreferenced_resources()
+        pdf.save(target_file, **save_settings)
 
     input_size = input_file.stat().st_size
     output_size = target_file.stat().st_size
@@ -660,9 +713,9 @@ def optimize(
             "optimizations will not be used"
         )
         # We still need to save the file
-        with Pdf.open(input_file) as pike:
-            pike.remove_unreferenced_resources()
-            pike.save(output_file, **save_settings)
+        with Pdf.open(input_file) as pdf:
+            pdf.remove_unreferenced_resources()
+            pdf.save(output_file, **save_settings)
     else:
         safe_symlink(target_file, output_file)
 
@@ -670,11 +723,12 @@ def optimize(
 
 
 def main(infile, outfile, level, jobs=1):
+    """Entry point for direct optimization of a file."""
     from shutil import copy  # pylint: disable=import-outside-toplevel
     from tempfile import TemporaryDirectory  # pylint: disable=import-outside-toplevel
 
     class OptimizeOptions:
-        """Emulate ocrmypdf's options"""
+        """Emulate ocrmypdf's options."""
 
         def __init__(
             self, input_file, jobs, optimize_, jpeg_quality, png_quality, jb2lossy
@@ -686,6 +740,7 @@ def main(infile, outfile, level, jobs=1):
             self.png_quality = png_quality
             self.jbig2_page_group_size = 0
             self.jbig2_lossy = jb2lossy
+            self.jbig2_threshold = 0.85
             self.quiet = True
             self.progress_bar = False
 
