@@ -6,11 +6,15 @@ from __future__ import annotations
 
 import logging
 import os
+import re
 import shutil
 import sys
+from collections.abc import Iterable, Iterator
 from itertools import chain
 from pathlib import Path
-from typing import Any, Callable, Iterable, Iterator, TypeVar
+from typing import Any, Callable, TypeVar
+
+from packaging.version import InvalidVersion, Version
 
 if sys.version_info >= (3, 10):
     from typing import TypeAlias
@@ -29,12 +33,13 @@ else:
         spec=['HKEYType', 'EnumKey', 'EnumValue', 'HKEY_LOCAL_MACHINE', 'OpenKey']
     )
     # mypy does not understand winreg.HKeyType where winreg is a Mock (fair enough!)
-    HKEYType: TypeAlias = Any
+    HKEYType: TypeAlias = Any  # type: ignore
 
 
 log = logging.getLogger(__name__)
 
 T = TypeVar('T')
+Tkey = TypeVar('Tkey')
 
 
 def ghostscript_version_key(s: str) -> tuple[int, int, int]:
@@ -99,6 +104,30 @@ def registry_path_tesseract(env=None) -> Iterator[Path]:
         log.warning(e)
 
 
+def _gs_version_in_path_key(path: Path) -> tuple[str, Version | None]:
+    """Key function for comparing Ghostscript and Tesseract paths.
+
+    Ghostscript installs on Windows:
+        %PROGRAMFILES%/gs/gs9.56.1/bin -> ('gs', Version('9.56.1'))
+        %PROGRAMFILES%/gs/9.24/bin -> ('gs', Version('9.24'))
+
+    Tesseract looks like:
+        %PROGRAMFILES%/Tesseract-OCR -> ('Tesseract-OCR', None)
+
+    Thus ensuring the resulting tuple will order the alternatives correctly,
+    e.g. gs10.0 > gs9.99.
+    """
+    match = re.search(r'gs[/\\]?([0-9.]+)[/\\]bin', str(path))
+    if match:
+        try:
+            version_str = match.group(1)
+            version = Version(version_str)
+            return 'gs', version
+        except InvalidVersion:
+            pass
+    return path.name, None
+
+
 def program_files_paths(env=None) -> Iterator[Path]:
     if not env:
         env = os.environ
@@ -116,7 +145,7 @@ def program_files_paths(env=None) -> Iterator[Path]:
     return iter(
         sorted(
             (p for p in path_walker()),
-            key=lambda p: (p.name, p.parent.name),
+            key=_gs_version_in_path_key,
             reverse=True,
         )
     )
@@ -141,13 +170,7 @@ SHIMS = [
 
 
 def fix_windows_args(program: str, args, env):
-    """Adjust our desired program and command line arguments for use on Windows"""
-
-    if sys.version_info < (3, 8):
-        # bpo-33617 - Windows needs manual Path -> str conversion
-        args = [os.fspath(arg) for arg in args]
-        program = os.fspath(program)
-
+    """Adjust our desired program and command line arguments for use on Windows."""
     # If we are running a .py on Windows, ensure we call it with this Python
     # (to support test suite shims)
     if program.lower().endswith('.py'):
@@ -164,11 +187,11 @@ def fix_windows_args(program: str, args, env):
     return args
 
 
-def unique_everseen(iterable: Iterable[T], key: Callable[[T], T]) -> Iterator[T]:
-    "List unique elements, preserving order."
+def unique_everseen(iterable: Iterable[T], key: Callable[[T], Tkey]) -> Iterator[T]:
+    """List unique elements, preserving order."""
     # unique_everseen('AAAABBBCCDAABBB') --> A B C D
     # unique_everseen('ABBCcAD', str.lower) --> A B C D
-    seen: set[T] = set()
+    seen: set[Tkey] = set()
     seen_add = seen.add
     for element in iterable:
         k = key(element)
@@ -177,11 +200,15 @@ def unique_everseen(iterable: Iterable[T], key: Callable[[T], T]) -> Iterator[T]
             yield element
 
 
+def _casefold_path(path: Path) -> str:
+    return str.casefold(str(path))
+
+
 def shim_env_path(env=None):
     if env is None:
         env = os.environ
 
     shim_paths = chain.from_iterable(shim(env) for shim in SHIMS)
     return os.pathsep.join(
-        str(p) for p in unique_everseen(shim_paths, key=lambda p: str.casefold(str(p)))
+        str(p) for p in unique_everseen(shim_paths, key=_casefold_path)
     )
