@@ -17,17 +17,18 @@ def raise_gs_fail(*args, **kwargs):
 
 
 @hookimpl
-def generate_pdfa(pdf_pages, pdfmark, output_file, compression, pdf_version, pdfa_part):
+def generate_pdfa(pdf_pages, pdfmark, output_file, context, pdf_version, pdfa_part):
     with patch('ocrmypdf._exec.ghostscript.run_polling_stderr') as mock:
         mock.side_effect = raise_gs_fail
         ghostscript.generate_pdfa(
             pdf_pages=pdf_pages,
             pdfmark=pdfmark,
             output_file=output_file,
-            compression=compression,
+            context=context,
             pdf_version=pdf_version,
             pdfa_part=pdfa_part,
             progressbar_class=None,
+            stop_on_soft_error=True,
         )
         mock.assert_called()
         return output_file
