@@ -178,7 +178,7 @@ def test_maximum_options(renderer, output_type, multipage, outpdf):
 
 
 @pytest.mark.skipif(
-    tesseract.TesseractVersion(tesseract.version()) >= tesseract.TesseractVersion('5'),
+    tesseract.version() >= tesseract.TesseractVersion('5'),
     reason="tess 5 tries harder to find its files",
 )
 def test_tesseract_missing_tessdata(monkeypatch, resources, no_outpdf, tmpdir):
@@ -446,7 +446,7 @@ def test_linearized_pdf_and_indirect_object(resources, outpdf):
 
 
 def test_very_high_dpi(resources, outpdf):
-    "Checks for a Decimal quantize error with high DPI, etc"
+    """Checks for a Decimal quantize error with high DPI, etc."""
     check_ocrmypdf(
         resources / '2400dpi.pdf',
         outpdf,
