@@ -42,7 +42,6 @@ VECTOR_RESOLUTION = Resolution(_pipeline.VECTOR_PAGE_DPI, _pipeline.VECTOR_PAGE_
     ],
 )
 def test_dpi_needed(image, text, vector, result, rgb_image, outdir):
-
     c = Canvas(str(outdir / 'dpi.pdf'), pagesize=(5 * inch, 5 * inch))
     if image:
         c.drawImage(rgb_image, 1 * inch, 1 * inch, width=1 * inch, height=1 * inch)
@@ -53,13 +52,14 @@ def test_dpi_needed(image, text, vector, result, rgb_image, outdir):
     c.showPage()
     c.save()
 
-    mock = Mock()
-    mock.oversample = DUMMY_OVERSAMPLE_RESOLUTION[0]
-
     pi = pdfinfo.PdfInfo(outdir / 'dpi.pdf')
+    pageinfo = pi[0]
+    ctx = Mock()
+    ctx.options.oversample = DUMMY_OVERSAMPLE_RESOLUTION[0]
+    ctx.pageinfo = pageinfo
 
-    assert _pipeline.get_canvas_square_dpi(pi[0], mock) == result
-    assert _pipeline.get_page_square_dpi(pi[0], mock) == result
+    assert _pipeline.get_canvas_square_dpi(ctx) == result
+    assert _pipeline.get_page_square_dpi(ctx) == result
 
 
 @pytest.mark.parametrize(
