@@ -58,7 +58,9 @@ def test_content_preservation(resources, outpdf):
     assert len(page.images) > 1, "masks were rasterized"
 
 
-@pytest.mark.skipif(tesseract.version() > '5', reason="doesn't fool Tess 5")
+@pytest.mark.skipif(
+    tesseract.version() > tesseract.TesseractVersion('5'), reason="doesn't fool Tess 5"
+)
 def test_no_languages(tmp_path, monkeypatch):
     (tmp_path / 'tessdata').mkdir()
     monkeypatch.setenv('TESSDATA_PREFIX', fspath(tmp_path))
